// -------------------------------------------------- //
//  Mad-Assembler by Tomasz Biela (aka TeBe/Madteam)  //
//  support 6502, 65816, .LOCAL, .MACRO, .PROC, SDX,  //
//  banks                                             //
//  last changes: 20.03.2005                          //
// -------------------------------------------------- //

program MADS;

{$APPTYPE CONSOLE}

type
    labels  = record
                nam: cardinal;   // identyfikator etykiety (CRC32)
                adr: cardinal;   // wartosc etykiety
                bnk: word;       // bank pamieci dla etykiety
                blk: word;       // blok przypisany etykiecie
                pas: byte;       // przebieg
                rel: Boolean;    // czy relokowac
              end;

    wywolyw = record
               zm: string;
               pl: string;
               nr: integer;
              end;

    stosife = record
                _elseif: Boolean;
               _if_test: Boolean;
                  _else: Boolean;
                _okelse: word;
              end;

   relocLab = record
                adr: integer;    // adres relokowalny
                idx: integer;    // indeks do T_SMB
                blk: word;       // numer bloku
                blo: word;
              end;

   relocSmb = record
                smb: string[8];    // 8 znakowa nazwa SMB
                use: Boolean;
              end;

   stctTab  = record
                 nr: integer;     // numer struktury
                ofs: word;        // ofset od poczatku struktury
                siz: word;        // dlugosc pola
                lab: string;      // etykieta wystepujaca w .STRUCT
              end;

    m64kb   = array [0..$FFFF] of byte;
    m512b   = array [0..511] of byte;
    t256w   = array [0..255] of word;
    t256c   = array [0..255] of cardinal;

var lst, lab, hhh: textfile;
    dst: file;

    pass, opt_old: byte;
    opt: byte = 3;
    cpu: byte = 8;

    help, bank, blok, nr_param, ifelse, buf_i, struct_adres: word;

    IORes, hea_i, rel_ofs, struct_nr: integer;
    proc_nr, lokal_nr, dek_macro_nr: integer;
    line, line_all, rept_line: integer;
    hea_ofs: integer = -1;
    adres: integer = -$FFFF;

    plik_lst, plik_obj, plik_lab, plik_asm, plik_h, plik_mac: string;
    path, name, t, global_name, proc_name: string;
    macro_nr: string = '';
    lokal_name: string = '';

    white_spaces: Boolean = false;
    label_used: Boolean = false;
    dta_used: Boolean = false;
    struct: Boolean = false;
    pisz: Boolean = false;
    rept: Boolean = false;
    empty: Boolean = false;
    reloc: Boolean = false;
    branch: Boolean = false;
    vector: Boolean = false;
    silent: Boolean = false;
    bez_lst: Boolean = false;
    icl_used: Boolean = false;
    komentarz: Boolean = false;
    case_used: Boolean = false;
    full_name: Boolean = false;
    exit_used: Boolean = false;
    else_used: Boolean = false;
    elseif_used: Boolean = false;
    macro: Boolean = false;
    proc: Boolean = false;
    run_macro: Boolean = false;
    loop_used: Boolean = false;
    dat: Boolean = false;
    org: Boolean = false;
    over: Boolean = false;
    open_ok: Boolean = false;
    list_lab: Boolean = false;
    list_hhh: Boolean = false;
    list_mac: Boolean = false;

    first_org: Boolean = true;
    if_test: Boolean = true;
    hea: Boolean = true;

    oblicz_mnemonik__: function(var i:word; var a,old:string): Int64;
    analizuj_plik__: procedure(var a:string; old_str: string);
    analizuj_mem__: procedure (start,koniec:integer; var old,a,old_str:string);

    tCRC16: t256w;
    tCRC32: t256c;

    // hashowane mnemoniki i tryby adresowania
    hash: m64kb;

    // bufor dla odczytu plikow przez INS
    // maksymalna dlugosc pliku to 64KB
    t_ins: m64kb;

    // bufor pamieci dla zapisu
    t_buf: m512b;

    // T_HEA zapamieta dlugosc bloku
    t_hea: array of word;

    // T_REL zapamieta relokowalne adresy etykiet
    t_rel: array of relocLab;

    // T_REL zapamieta relokowalne adresy symboli
    t_smb: array of relocSmb;

    // T_PRC zapamieta liczbe zadeklarowanych parametrow
    t_prc: array of integer;

    // T_MAC zapamieta linie z makrami
    t_mac: array of string;

    // T_LOC zapamieta nazwy obszarow .LOCAL
    t_loc: array of string;

    // T_PTH zapamieta sciezki poszukiwan dla INS i ICL
    t_pth: array of string;

    // T_LAB zapamieta etykiety
    t_lab: array of labels;

    // T_WYW zapamieta linie z wywolywanymi makrami
    t_wyw: array of wywolyw;

    // T_WYW zapamieta linie z wywolywanymi makrami
    t_str: array of stctTab;

    // stos dla operacji .IF .ELSE .ENDIF
    if_stos: array of stosife;

const
  __macro  = $A0;  // kolejnosc wg tablicy 'MAC' + $A0
  __if     = $A1;
  __endif  = $A2;
  __endm   = $A3;
  __exit   = $A4;
  __error  = $A5;
  __else   = $A6;
  __print  = $A7;
  __proc   = $A8;
  __endp   = $A9;
  __elseif = $AA;
  __local  = $AB;
  __endl   = $AC;
  __rept   = $AD;
  __endr   = $AE;

  __byte   = $AF;
  __word   = $B0;
  __long   = $B1;
  __dword  = $B2;

  __struct = $B3;
  __ends   = $B4;
  __ds     = $B5;

  __over   = __ds;

  __nill   = $E0;

  __rel    = $0000;           // wartosc dla etykiet relokowalnych

  __id_param  = $FFFB;
  __id_smb    = $FFFC;
  __id_macro  = $FFFD;
  __id_struct = $FFFE;
  __id_proc   = $FFFF;

  __macro_run  = lo(__id_macro);
  __struct_run = lo(__id_struct);
  __proc_run   = lo(__id_proc);

 pass_end = 3;       // pass = <0..3>

{
 tCRC32: array [0..255] of cardinal=(
 $00000000,$77073096,$ee0e612c,$990951ba,$076dc419,$706af48f,$e963a535,$9e6495a3,
 $0edb8832,$79dcb8a4,$e0d5e91e,$97d2d988,$09b64c2b,$7eb17cbd,$e7b82d07,$90bf1d91,
 $1db71064,$6ab020f2,$f3b97148,$84be41de,$1adad47d,$6ddde4eb,$f4d4b551,$83d385c7,
 $136c9856,$646ba8c0,$fd62f97a,$8a65c9ec,$14015c4f,$63066cd9,$fa0f3d63,$8d080df5,
 $3b6e20c8,$4c69105e,$d56041e4,$a2677172,$3c03e4d1,$4b04d447,$d20d85fd,$a50ab56b,
 $35b5a8fa,$42b2986c,$dbbbc9d6,$acbcf940,$32d86ce3,$45df5c75,$dcd60dcf,$abd13d59,
 $26d930ac,$51de003a,$c8d75180,$bfd06116,$21b4f4b5,$56b3c423,$cfba9599,$b8bda50f,
 $2802b89e,$5f058808,$c60cd9b2,$b10be924,$2f6f7c87,$58684c11,$c1611dab,$b6662d3d,
 $76dc4190,$01db7106,$98d220bc,$efd5102a,$71b18589,$06b6b51f,$9fbfe4a5,$e8b8d433,
 $7807c9a2,$0f00f934,$9609a88e,$e10e9818,$7f6a0dbb,$086d3d2d,$91646c97,$e6635c01,
 $6b6b51f4,$1c6c6162,$856530d8,$f262004e,$6c0695ed,$1b01a57b,$8208f4c1,$f50fc457,
 $65b0d9c6,$12b7e950,$8bbeb8ea,$fcb9887c,$62dd1ddf,$15da2d49,$8cd37cf3,$fbd44c65,
 $4db26158,$3ab551ce,$a3bc0074,$d4bb30e2,$4adfa541,$3dd895d7,$a4d1c46d,$d3d6f4fb,
 $4369e96a,$346ed9fc,$ad678846,$da60b8d0,$44042d73,$33031de5,$aa0a4c5f,$dd0d7cc9,
 $5005713c,$270241aa,$be0b1010,$c90c2086,$5768b525,$206f85b3,$b966d409,$ce61e49f,
 $5edef90e,$29d9c998,$b0d09822,$c7d7a8b4,$59b33d17,$2eb40d81,$b7bd5c3b,$c0ba6cad,
 $edb88320,$9abfb3b6,$03b6e20c,$74b1d29a,$ead54739,$9dd277af,$04db2615,$73dc1683,
 $e3630b12,$94643b84,$0d6d6a3e,$7a6a5aa8,$e40ecf0b,$9309ff9d,$0a00ae27,$7d079eb1,
 $f00f9344,$8708a3d2,$1e01f268,$6906c2fe,$f762575d,$806567cb,$196c3671,$6e6b06e7,
 $fed41b76,$89d32be0,$10da7a5a,$67dd4acc,$f9b9df6f,$8ebeeff9,$17b7be43,$60b08ed5,
 $d6d6a3e8,$a1d1937e,$38d8c2c4,$4fdff252,$d1bb67f1,$a6bc5767,$3fb506dd,$48b2364b,
 $d80d2bda,$af0a1b4c,$36034af6,$41047a60,$df60efc3,$a867df55,$316e8eef,$4669be79,
 $cb61b38c,$bc66831a,$256fd2a0,$5268e236,$cc0c7795,$bb0b4703,$220216b9,$5505262f,
 $c5ba3bbe,$b2bd0b28,$2bb45a92,$5cb36a04,$c2d7ffa7,$b5d0cf31,$2cd99e8b,$5bdeae1d,
 $9b64c2b0,$ec63f226,$756aa39c,$026d930a,$9c0906a9,$eb0e363f,$72076785,$05005713,
 $95bf4a82,$e2b87a14,$7bb12bae,$0cb61b38,$92d28e9b,$e5d5be0d,$7cdcefb7,$0bdbdf21,
 $86d3d2d4,$f1d4e242,$68ddb3f8,$1fda836e,$81be16cd,$f6b9265b,$6fb077e1,$18b74777,
 $88085ae6,$ff0f6a70,$66063bca,$11010b5c,$8f659eff,$f862ae69,$616bffd3,$166ccf45,
 $a00ae278,$d70dd2ee,$4e048354,$3903b3c2,$a7672661,$d06016f7,$4969474d,$3e6e77db,
 $aed16a4a,$d9d65adc,$40df0b66,$37d83bf0,$a9bcae53,$debb9ec5,$47b2cf7f,$30b5ffe9,
 $bdbdf21c,$cabac28a,$53b39330,$24b4a3a6,$bad03605,$cdd70693,$54de5729,$23d967bf,
 $b3667a2e,$c4614ab8,$5d681b02,$2a6f2b94,$b40bbe37,$c30c8ea1,$5a05df1b,$2d02ef8d);
}

 maska:array [1..23] of cardinal=(
 $01,$02,$04,$08,$10,$20,$40,$80,$100,$200,$400,$800,$1000,$2000,
 $4000,$8000,$10000,$20000,$40000,$80000,$100000,$200000,$400000);

 addycja: array [1..24] of byte=(
 0,0,4,8,8,12,16,20,20,24,28,44,0,0,8,4,8,16,20,24,24,32,32,32);

 addycja_16: array [1..207] of byte=(
 0,0,0,2,2,4,6,8,8,12,14,16,17,18,20,20,22,24,28,30,32,34,48,
 0,0,0,2,2,8,6,4,8,16,14,16,17,18,20,24,22,32,28,30,32,34,48,
 0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,220,
 0,0,0,0,0,0,0,0,0,0,16,0,0,0,0,0,0,0,0,0,32,144,48,
 0,0,0,0,0,0,0,101,0,8,0,0,0,0,16,0,0,0,24,0,0,0,0,
 0,0,0,0,0,0,0,0,0,56,0,0,0,0,16,0,0,0,58,0,0,0,0,
 0,0,0,0,0,0,0,146,0,0,0,0,114,0,0,0,0,0,0,0,0,0,0,
 0,0,0,0,0,140,0,0,0,148,0,0,0,0,156,0,0,0,164,0,0,0,0,
 0,0,0,0,0,204,0,0,0,212,0,0,0,0,220,0,0,0,228,0,0,0,0);

 // kod maszynowy mnemonika w pierwszej adresacji (CPU 6502)
 kod: array [0..55] of byte=(
 $A1,$9E,$9C,$81,$82,$80,$61,$21,
 $02,$E1,$14,$40,$42,$01,$C1,$BC,
 $DC,$C2,$E2,$41,$22,$62,$00,$18,
 $58,$B8,$D8,$08,$28,$48,$68,$40,
 $60,$38,$78,$F8,$C8,$E8,$88,$CA,
 $8A,$98,$9A,$A8,$AA,$BA,$EA,$10,
 $30,$D0,$90,$B0,$F0,$50,$70,$20);

 // kod maszynowy mnemonika w pierwszej adresacji (CPU 65816)
 kod_16: array [0..88] of byte=(
 $A1,$9E,$9C,$81,$82,$80,$61,$21,
 $02,$E1,$20,$4C,$42,$01,$C1,$BC,
 $DC,$3A,$1A,$41,$22,$62,$00,$18,
 $58,$B8,$D8,$08,$28,$48,$68,$40,
 $60,$38,$78,$F8,$C8,$E8,$88,$CA,
 $8A,$98,$9A,$A8,$AA,$BA,$EA,$10,
 $30,$D0,$90,$B0,$F0,$50,$70,$24,
 $64,$DE,$BE,$10,$00,$80,$FE,$54,
 $44,$62,$8B,$0B,$4B,$DA,$5A,$AB,
 $2B,$FA,$7A,$6B,$DB,$5B,$1B,$7B,
 $3B,$9B,$BB,$CB,$42,$EB,$FB,$3A,
 $1A);

 // mozliwe adresacje rozkazu (10 bitow) CPU6502
 // bity w najstarszym bajcie oznaczaja przesuniecie w tablicy 'ADDYCJA'
 ads: array [0..55] of cardinal=(
 $000006ED,$8000032C,$800004AC,$000006E5,$00000124,$000000A4,$000006ED,$000006ED,
 $000004B4,$000006ED,$00000020,$00000820,$000004B4,$000006ED,$000006ED,$8000002C,
 $8000002C,$000004A4,$000004A4,$000006ED,$000004B4,$000004B4,$00000000,$00000000,
 $00000000,$00000000,$00000000,$00000000,$00000000,$00000000,$00000000,$00000000,
 $00000000,$00000000,$00000000,$00000000,$00000000,$00000000,$00000000,$00000000,
 $00000000,$00000000,$00000000,$00000000,$00000000,$00000000,$00000000,$00000002,
 $00000002,$00000002,$00000002,$00000002,$00000002,$00000002,$00000002,$00000024);

 // mozliwe adresacje rozkazu (32 bity) CPU65816
 // bity w najstarszym bajcie oznaczaja przesuniecie w tablicy 'ADDYCJA_16'
 ads_16: array [0..88] of cardinal=(
 $000F7EE9,$800282A0,$800282A0,$000F7E69,$00008220,$00004220,$000F7EE9,$000F7EE9,
 $00044320,$000F7EE9,$40400600,$20700600,$00044320,$000F7EE9,$000F7EE9,$800002A0,
 $800002A0,$02044320,$01044320,$000F7EE9,$00044320,$00044320,$00000000,$00000000,
 $00000000,$00000000,$00000000,$00000000,$00000000,$00000000,$00000000,$00000000,
 $00000000,$00000000,$00000000,$00000000,$00000000,$00000000,$00000000,$00000000,
 $00000000,$00000000,$00000000,$00000000,$00000000,$00000000,$00000000,$00000002,
 $00000002,$00000002,$00000002,$00000002,$00000002,$00000002,$00000002,$100442A0,
 $08044220,$80000080,$80000080,$00000220,$00000220,$00000012,$80000080,$00000004,
 $00000004,$04001280,$00000000,$00000000,$00000000,$00000000,$00000000,$00000000,
 $00000000,$00000000,$00000000,$00000000,$00000000,$00000000,$00000000,$00000000,
 $00000000,$00000000,$00000000,$00000000,$00000000,$00000000,$00000000,$00000000,
 $00000000);

 // HASH_VAL i HASH_OFS wylicza osobny program
 // nie trzeba przechowywac w pamieci stringow z nazwami
 hash_val: array [0..165] of byte=(
 $01,$02,$03,$04,$05,$06,$07,$08,$09,$0A,$0B,$0C,$01,$02,$03,$04,
 $05,$06,$07,$08,$09,$0A,$0B,$0C,$0D,$0E,$0F,$10,$11,$12,$13,$14,
 $15,$16,$17,$A0,$A1,$A2,$A3,$A4,$A5,$A6,$A7,$A8,$A9,$AA,$AB,$AC,
 $AD,$AE,$AF,$B0,$B1,$B2,$B3,$B4,$B5,$81,$82,$83,$84,$85,$86,$87,
 $88,$89,$8A,$8B,$8C,$8D,$8E,$8F,$90,$91,$92,$93,$94,$01,$02,$03,
 $04,$05,$06,$07,$08,$09,$0A,$0B,$0C,$0D,$0E,$0F,$10,$11,$12,$13,
 $14,$15,$16,$17,$18,$19,$1A,$1B,$1C,$1D,$1E,$1F,$20,$21,$22,$23,
 $24,$25,$26,$27,$28,$29,$2A,$2B,$2C,$2D,$2E,$2F,$30,$31,$32,$33,
 $34,$35,$36,$37,$38,$39,$3A,$3B,$3C,$3D,$3E,$3F,$40,$41,$42,$43,
 $44,$45,$46,$47,$48,$49,$4A,$4B,$4C,$4D,$4E,$4F,$50,$51,$52,$53,
 $54,$55,$56,$57,$58,$59);

 hash_ofs: array [0..165] of word=(
 $9F52,$8976,$1A4F,$F5F1,$A934,$AB24,$7833,$61A6,$5297,$E3FC,$D0CD,$D894,$3721,$5EDF,$155E,$AFC6,
 $1C4B,$CDE6,$266F,$2258,$7E9D,$7C8D,$2C28,$D040,$7B65,$CA5A,$733C,$400D,$8D4A,$F166,$C257,$92F2,
 $CA0E,$9704,$864A,$068A,$CE0D,$1B79,$5A04,$5E83,$463C,$2F88,$8154,$F6D0,$9998,$990E,$68C2,$4A25,
 $ACD0,$B9DA,$4FA1,$A307,$D0C2,$56C2,$E073,$A9FB,$FAC5,$0D1F,$E9CE,$ADFE,$0BF5,$1C42,$976F,$9E77,
 $A472,$D926,$B88E,$B8D2,$EF24,$B746,$486D,$F2BB,$18F6,$A597,$AB8D,$D814,$0E5F,$3DBD,$BEA5,$AE84,
 $519C,$D284,$C2A5,$5FAE,$C082,$34A5,$D80B,$37AB,$3795,$850B,$CD38,$A904,$4D02,$5D23,$876F,$19C4,
 $5D84,$6888,$9B77,$2E23,$B867,$192D,$FAF3,$C880,$4CC2,$8006,$4ED2,$8216,$E7A4,$54DF,$419C,$E0D6,
 $317B,$AABF,$BA9E,$3414,$2435,$9161,$A250,$A312,$BBB3,$AB92,$CE83,$8D37,$38A6,$1D2C,$89F3,$9F69,
 $8D58,$07BC,$63EF,$71DE,$1274,$F2C6,$63CE,$54FE,$4EC9,$7DF8,$8F69,$CF66,$9E73,$6D8C,$388E,$7EB1,
 $1E77,$EF98,$CDCA,$DDEB,$B275,$D2B3,$010E,$112F,$B701,$538C,$1E4D,$7C9B,$F73D,$6DD9,$0258,$2148,
 $F0D2,$4FA3,$08B8,$7B0D,$A72D,$3986);

 // komunikaty
 mes: array [0..1459] of char=
{0} (chr(ord('V') + $80),'a','l','u','e',' ','o','u','t',' ','o','f',' ','r','a','n','g','e',
{1}  chr(ord('M') + $80),'i','s','s','i','n','g',' ','.','E','N','D','I','F',
{2}  chr(ord('L') + $80),'a','b','e','l',' ','d','e','c','l','a','r','e','d',' ','t','w','i','c','e',
{3}  chr(ord('S') + $80),'t','r','i','n','g',' ','e','r','r','o','r',
{4}  chr(ord('E') + $80),'x','t','r','a',' ','c','h','a','r','a','c','t','e','r','s',' ','o','n',' ','l','i','n','e',
{5}  chr(ord('U') + $80),'n','d','e','c','l','a','r','e','d',' ','l','a','b','e','l',' ',
{6}  chr(ord('N') + $80),'o',' ','m','a','t','c','h','i','n','g',' ','b','r','a','c','k','e','t',
{7}  chr(ord('N') + $80),'e','e','d',' ','p','a','r','e','n','t','h','e','s','i','s',
{8}  chr(ord('I') + $80),'l','l','e','g','a','l',' ','c','h','a','r','a','c','t','e','r',
{9}  chr(ord('R') + $80),'e','s','e','r','v','e','d',' ','w','o','r','d',
{10} chr(ord('N') + $80),'o',' ','O','R','G',' ','s','p','e','c','i','f','i','e','d',
{11} chr(ord('U') + $80),'s','e',' ','s','q','u','a','r','e',' ','b','r','a','c','k','e','t','s',' ','i','n','s','t','e','a','d',
{12} chr(ord('I') + $80),'l','l','e','g','a','l',' ','i','n','s','t','r','u','c','t','i','o','n',' ',
{13} chr(ord('S') + $80),'p','a','c','e',' ','e','x','p','e','c','t','e','d',
{14} chr(ord('I') + $80),'l','l','e','g','a','l',' ','a','d','r','e','s','s','i','n','g',' ','m','o','d','e',' ','(','C','P','U',' ','6','5',
{15} chr(ord('L') + $80),'a','b','e','l',' ','n','a','m','e',' ','r','e','q','u','i','r','e','d',
{16} chr(ord('I') + $80),'n','v','a','l','i','d',' ','o','p','t','i','o','n',
{17} chr(ord('M') + $80),'i','s','s','i','n','g',' ','.','E','N','D',
{18} chr(ord('C') + $80),'a','n','''','t',' ','o','p','e','n',' ','f','i','l','e',
{19} chr(ord('N') + $80),'e','s','t','e','d',' ','o','p','-','c','o','d','e','s',' ','n','o','t',' ','s','u','p','p','o','r','t','e','d',
{20} chr(ord('M') + $80),'i','s','s','i','n','g',' ','''','}','''',
{21} chr(ord('B') + $80),'r','a','n','c','h',' ','o','u','t',' ','o','f',' ','r','a','n','g','e',' ','b','y',' ','$',
{22} chr(ord(' ') + $80),'b','y','t','e','s',
{23} chr(ord('U') + $80),'n','e','x','p','e','c','t','e','d',' ','e','o','l',
{24} chr(ord('F') + $80),'i','l','e',' ','i','s',' ','t','o','o',' ','s','h','o','r','t',
{25} chr(ord('F') + $80),'i','l','e',' ','i','s',' ','t','o','o',' ','l','o','n','g',
{26} chr(ord('D') + $80),'i','v','i','d','e',' ','b','y',' ','z','e','r','o',
{27} chr(ord('S') + $80),'y','n','t','a','x',':',' ','M','A','D','S',' ','s','o','u','r','c','e',' ','[','o','p','t','i','o','n','s',']',
{28} chr(ord('M') + $80),'i','s','s','i','n','g',' ','.','L','O','C','A','L',
{29} chr(ord('M') + $80),'i','s','s','i','n','g',' ','.','E','N','D','L',
{30} chr(ord('U') + $80),'s','e','r',' ','e','r','r','o','r',
{31} chr(ord('O') + $80),'p','e','r','a','n','d',' ','o','v','e','r','f','l','o','w',
{32} chr(ord('B') + $80),'a','d',' ','s','i','z','e',' ','s','p','e','c','i','f','i','e','r',
{33} chr(ord('S') + $80),'i','z','e',' ','s','p','e','c','i','f','i','e','r',' ','n','o','t',' ','r','e','q','u','i','r','e','d',
{34} chr(ord(' ') + $80),
{35} chr(ord('U') + $80),'n','d','e','c','l','a','r','e','d',' ','m','a','c','r','o',' ',
{36} chr(ord('C') + $80),'a','n','''','t',' ','r','e','p','e','a','t',' ','t','h','i','s',' ','d','i','r','e','c','t','i','v','e',
{37} chr(ord('M') + $80),'i','s','s','i','n','g',' ','.','I','F',
{38} chr(ord('L') + $80),'a','b','e','l',' ','n','o','t',' ','r','e','q','u','i','r','e','d',
{39} chr(ord('U') + $80),'n','d','e','c','l','a','r','e','d',' ','p','r','o','c','e','d','u','r','e',
{40} chr(ord('I') + $80),'m','p','r','o','p','e','r',' ','n','u','m','b','e','r',' ','o','f',' ','a','c','t','u','a','l',' ','p','a','r','a','m','e','t','e','r','s',
{41} chr(ord('I') + $80),'n','c','o','m','p','a','t','i','b','l','e',' ','t','y','p','e','s',
{42} chr(ord('.') + $80),'E','N','D','I','F',' ','e','x','p','e','c','t','e','d',
{43} chr(ord('M') + $80),'i','s','s','i','n','g',' ','.','E','N','D','R',
{44} chr(ord('S') + $80),'M','B',' ','l','a','b','e','l',' ','t','o','o',' ','l','o','n','g',
{45} chr(ord('T') + $80),'o','o',' ','m','a','n','y',' ','b','l','o','c','k','s',
{46} chr(ord('B') + $80),'a','d',' ','p','a','r','a','m','e','t','e','r',' ','t','y','p','e',
{47} chr(ord('B') + $80),'a','d',' ','p','a','r','a','m','e','t','e','r',' ','n','u','m','b','e','r',
{48} chr(ord(' ') + $80),'l','i','n','e','s',' ','o','f',' ','s','o','u','r','c','e',' ','a','s','s','e','m','b','l','e','d',
{49} chr(ord(' ') + $80),'b','y','t','e','s',' ','w','r','i','t','t','e','n',' ','t','o',' ','t','h','e',' ','o','b','j','e','c','t',' ','f','i','l','e',
{50} chr(ord('M') + $80),'a','d','-','A','s','s','e','m','b','l','e','r',' ','v','1','.','6','.','0','b','e','t','a',' ','b','y',' ','T','e','B','e','/','M','a','d','t','e','a','m',
{51} chr(ord('M') + $80),'i','s','s','i','n','g',' ','.','R','E','P','T',
{52} chr(ord('B') + $80),'a','d',' ','o','r',' ','m','i','s','s','i','n','g',' ','s','i','n','u','s',' ','p','a','r','a','m','e','t','e','r',
{53} chr(ord('O') + $80),'n','l','y',' ','R','E','L','O','C',' ','b','l','o','c','k',
{54} chr(ord('L') + $80),'a','b','e','l',' ','t','a','b','l','e',':',
{55} chr(ord('M') + $80),'i','s','s','i','n','g',' ','.','S','T','R','U','C','T',
{56} chr(ord('M') + $80),'i','s','s','i','n','g',' ','.','E','N','D','S',
{57} chr(ord('C') + $80),'a','n',' ','n','o','t',' ','u','s','e',' ','r','e','c','u','r','s','i','v','e',' ','s','t','r','u','c','t','u','r','e','s',
{58} chr(ord('I') + $80),'m','p','r','o','p','e','r',' ','s','y','n','t','a','x',

{59} chr(ord('m') + $80),'a','d','s',' ','1','.','6','.','0',#13,#10,
     'S','y','n','t','a','x',':',' ','M','A','D','S',' ','s','o','u','r','c','e',' ','[','o','p','t','i','o','n','s',']',#13,#10,
     '/','c',#9,#9,'L','a','b','e','l',' ','c','a','s','e',' ','s','e','n','s','i','t','i','v','i','t','y',#13,#10,
     '/','e',#9,#9,'E','a','t',' ','w','h','i','t','e',' ','s','p','a','c','e','s',#13,#10,
     '/','h','[',':','f','i','l','e','n','a','m','e',']',#9,'H','e','a','d','e','r',' ','f','i','l','e',' ','f','o','r',' ','C','C','6','5',#13,#10,
     '/','i',':','p','a','t','h',#9,#9,'A','d','d','i','t','i','o','n','a','l',' ','i','n','c','l','u','d','e',' ','d','i','r','e','c','t','o','r','i','e','s',#13,#10,
     '/','l','[',':','f','i','l','e','n','a','m','e',']',#9,'G','e','n','e','r','a','t','e',' ','l','i','s','t','i','n','g',#13,#10,
     '/','m',':','f','i','l','e','n','a','m','e',#9,'F','i','l','e',' ','w','i','t','h',' ','m','a','c','r','o',' ','d','e','f','i','n','i','t','i','o','n',#13,#10,
     '/','o',':','f','i','l','e','n','a','m','e',#9,'S','e','t',' ','o','b','j','e','c','t',' ','f','i','l','e',' ','n','a','m','e',#13,#10,
     '/','p',#9,#9,'P','r','i','n','t',' ','f','u','l','l','y',' ','q','u','a','l','i','f','i','e','d',' ','f','i','l','e',' ','n','a','m','e','s',' ','i','n',' ','l','i','s','t','i','n','g',#13,#10,
     '/','s',#9,#9,'S','i','l','e','n','t',' ','m','o','d','e',#13,#10,
     '/','t','[',':','f','i','l','e','n','a','m','e',']',#9,'L','i','s','t',' ','l','a','b','e','l',' ','t','a','b','l','e',

     chr($80) );

 tHex: array [0..15] of char =
 ('0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F');


procedure omin_spacje (var i:word; var a:string);
begin
 if a<>'' then
  while a[i]=' ' do inc(i);
end;


procedure __inc(var x:word; var a:string);
begin
 inc(x);
 omin_spacje(x,a);
end;


function UpCas_(a: char):char;
begin
 Result:=a;

 if not(case_used) then Result:=UpCase(a);
end;


function ata2int(a: byte):byte;
// ATASCII -> INTERNAL
begin
Result:=a;

 case (a and $7f) of
    0..31: inc(Result,64);
  96..255: exit;
 else
  dec(Result,32);
 end;

end;


function IntToStr(a: integer):string;
begin
 str(a,Result);
end;

function StrToInt(var a: string):integer;
var err:integer;
begin
 val(a,Result,err);
end;


procedure flush_dst;
begin
 if buf_i>0 then blockwrite(dst,t_buf,buf_i);
 buf_i:=0;
end;


procedure justuj;
var i, j: word;
begin
 if (pass=pass_end) and (t<>'') then begin
  j:=length(t);
  while j<32 do begin
   i:=8-j mod 8; t:=t+#9; inc(j,i);
  end;
 end;
end;


function bHex(var a: byte):string;
begin
 Result:=tHex[a shr 4]+tHex[a and $0f];
end;

function Hex(a: Int64; b: byte):string;
var v: byte;
begin
 v:=a and $FF; Result:=bHex(v); if (b=2) and (a shr 8=0) then exit;
 v:=a shr 8; Result:=bHex(v)+Result;

 v:=a shr 16; if a>$FFFF then Result:=bHex(v)+Result;
 v:=a shr 24; if a>$FFFFFF then Result:=bHex(v)+Result;
end;


function load_mes(b: byte):string;
// tworzy string z komunikatem nr 'B'
var i:word;
begin
 i:=0;
 while b>0 do begin if ord(mes[i])>$7f then dec(b); inc(i) end;
 Result:=mes[i-1]; dec(Result[1],$80);

 while ord(mes[i])<$80 do begin Result:=Result+mes[i]; inc(i) end;
end;


procedure koniec(err: byte);
var a, b: integer;
    g: file;
    f: textfile;
    ok: Boolean;
begin
 if open_ok then begin
  flush(lst); closefile(lst);
  flush_dst; closefile(dst);

 // sprawdzamy dlugosc 'PLIK_OBJ'
  assignfile(g,plik_obj); reset(g,1);
  a:=FileSize(g); closefile(g);

  // sprawdzamy liczbe wierszy w 'PLIK_LST'
  assignfile(f,plik_lst); reset(f);
  ok:=true; b:=0;
  while (not eof(f)) and (ok=true)  do begin
   readln(f,t); inc(b);
   if b>2 then ok:=false;
  end;
  closefile(f);

  if list_lab then closefile(lab);

  if (a=0) or (err<>0) then Erase(dst);
  if (b<3) or (err<>0) then Erase(lst);

  if over and not(silent) then begin
   writeln(line_all,load_mes(48+1));
   write(a,load_mes(49+1));
  end;
 end;

 if list_hhh then begin
  writeln(hhh,#13+#10+'#endif');
  closefile(hhh);
 end;

 halt(err);
end;


function GetFileName(var a: string):string;
var i:word;
begin
// Result:=ExtractFileName(a);
 i:=length(a);
 while (a[i]<>'\') and (i>=1) do dec(i);

 Result:=copy(a,i+1,length(a));
end;


function GetFilePath(var a: string):string;
var i:word;
begin
 Result:='';

 if a<>'' then begin
  i:=length(a);
  while (a[i]<>'\') and (i>=1) do dec(i);

  Result:=copy(a,1,i);
 end;
end;


function TestFile(var a: string):Boolean;
var pl: textfile;
begin
 Result:=true;

 AssignFile(pl, a);
 {$I-}
 Reset(pl);
 {$I+ }
 IORes:=IOResult;
 if not((IORes=32) or (IORes=0)) then Result:=false else CloseFile(pl);
end;


function GetFile(var a: string):string;
// szukamy pliku w zadeklarowanych sciezkach poszukiwan
var n, p, c: string;
    i: integer;
begin
 p:=GetFilePath(a);
 if p<>'' then
  if p[2]<>':' then a:=path+a;

 Result:=a;

 if TestFile(a) then exit;

 n:=GetFileName(a);

 for i:=0 to High(t_pth) do begin
  p:=t_pth[i];

  if p<>'' then
    if p[length(p)]<>'\' then p:=p+'\';

  c:=p+n;
  if TestFile(c) then begin Result:=c; exit end;
 end;

end;


function show_full_name(var a:string; b,c:Boolean):string;
var pth:string;
begin
 pth:=GetFilePath(a);

 Result:=a;

 if b then begin

  if pth='' then Result:=path+Result;

 end else Result:=GetFileName(Result);

 if c then Result:='Source: '+Result;
end;


procedure warning(a: byte);
begin
 writeln(show_full_name(global_name,full_name,false)+' ('+IntToStr(line)+') WARNING: '+load_mes(a+1));
end;


procedure blad(var a:string; b:integer);
// wyswietla komunikat bledu nr 'B'
// wyjatek stanowi b<0 wtedy wyswietli komunikat 'Branch...'
// dla komunikatu nr 14 wyswietli nazwe ustawionego CPU
var add, prv: string;
begin
 add:=''; prv:='';

 if run_macro then begin
  writeln(t_wyw[0].zm);
  global_name:=t_wyw[0].pl;
  inc(line,t_wyw[0].nr);
 end else
  if rept_line>0 then inc(line,rept_line);

 while pos(#0,a)>0 do a[pos(#0,a)]:=#1;

 if (a<>'') and (b<>34) then writeln(a);

 write(show_full_name(global_name,full_name,false)+' ('+IntToStr(line)+') ERROR: ');

 if b in [5,8,35] then add:=lokal_name;
 if b in [2,5,35,39] then add:=add+' (BANK='+IntToStr(bank)+')';

 if b=14 then
  if opt and 16>0 then add:='816)' else add:='02)';

 if b=17 then
  if macro then add:='M' else
   if proc then add:='P';

 if b<0 then writeln(load_mes(22)+Hex(abs(b),4)+load_mes(23))
  else if b<>34 then writeln(prv+load_mes(b+1)+add)
   else writeln(a+add);

 koniec(2);
end;


procedure blad_ill(var a:string; c,d:char);
begin
 lokal_name:=' '+c+d;
 blad(a,8);
end;


function fCRC(var a: string): cardinal;
var i: word;
    b: byte;
begin
 Result:=$ffffffff;
 for i:=1 to length(a) do begin
  b:=Result xor ord(a[i]);
  Result:=tCRC32[b] xor ((Result shr 8) and $00ffffff);
 end;
end;


function fCRC16(var a: string): word;
var i: word;
    b: byte;
begin
 Result:=$FFFF;
 for i:=1 to length(a) do begin
  b:=(Result shr 8) xor ord(a[i]);
  Result:=tCRC16[b] xor (Result shl 8);
 end;
end;


function l_lab(a:string): integer;
//
// szuka etykiety i zwraca jej indeks do tablicy 'T_LAB'
// jesli nie ma takiej etykiety zwraca -1
//
var x: cardinal;
    i: integer;
begin
 Result:=-1;

 x:=fCRC(a);

// OK jesli znaleziona etykieta ma kod >=__id_param, lub aktualna wartosc BANK=0
// OK jesli znaleziona etykieta jest z aktualnego banku lub banku zerowego (BNK=0)

 for i:=High(t_lab) downto 0 do
  if t_lab[i].nam=x then
   if (bank=0) or (t_lab[i].bnk>=__id_param) then Result:=i else
    if t_lab[i].bnk in [bank,0] then Result:=i;

end;


function search(var a:string; x:string):integer;
var b: string;
    i: word;
begin
 b:=x+lokal_name;

 Result:=l_lab(b+a);

 while (Result<0) and (pos('.',b)>0) do begin
//  obetnij_ciag(b);

  i:=length(b);
  if (b<>'') and (i>1) then begin
   dec(i);
   while (b[i]<>'.') and (i>=1) do dec(i);

   b:=copy(b,1,i);
  end;

  Result:=l_lab(b+a);
 end;

end;


function load_lab(var a:string; test:Boolean): integer;
// szuka etykiety i zwraca jej indeks do tablicy 'T_LAB'
// jesli nie ma takiej etykiety zwraca -1
//
// jesli jest uruchomione makro i nie znajdzie etykiety to szukaj w proc
// jesli nie znajdzie i w proc to wtedy szukaj w glownym programie
begin

  if test then begin

    if macro then begin
     Result:=l_lab(lokal_name+a+'::'+IntToStr(dek_macro_nr));

     if Result>=0 then exit {else begin
      b:=lokal_name;
      while (Result<0) and (pos('.',b)>0) do begin
       b:=obetnij_ciag(b);
       Result:=l_lab(b+a+'::'+IntToStr(dek_macro_nr));
      end;
     end;

     if Result>=0 then exit;}
    end;


    if run_macro then begin
{     Result:=l_lab(macro_nr+lokal_name+a);

     if Result>=0 then exit else begin
      b:=macro_nr+lokal_name;
      while (Result<0) and (pos('.',b)>0) do begin
       obetnij_ciag(b);
       Result:=l_lab(b+a);
      end;

     end;}

     Result:=search(a,macro_nr);

     if Result>=0 then exit;
    end;


    if proc then begin
{     Result:=l_lab(proc_name+lokal_name+a);

     if Result>=0 then exit else begin
      b:=proc_name+lokal_name;
      while (Result<0) and (pos('.',b)>0) do begin
       obetnij_ciag(b);
       Result:=l_lab(b+a);
      end;
     end;}

     Result:=search(a,proc_name);

     if Result>=0 then exit;
    end;


{     Result:=l_lab(lokal_name+a);

     b:=lokal_name;
     while (Result<0) and (pos('.',b)>0) do begin
      obetnij_ciag(b);
      Result:=l_lab(b+a);
     end;}

    Result:=search(a,'');

    if Result>=0 then exit
  end;


 Result:=l_lab(lokal_name+a);

 if Result>=0 then exit else begin

{  b:=proc_name+lokal_name;
  while (Result<0) and (pos('.',b)>0) do begin
   obetnij_ciag(b);
   Result:=l_lab(b+a);
  end;}

  Result:=search(a,proc_name);
 end;

end;


procedure zapisz_etykiete(var a:string; ad:cardinal; ba:word; symbol:char);
begin
 if pass=pass_end then
  if symbol<>'?' then begin
   if list_lab then writeln(lab,Hex(ba,2)+#9+Hex(ad,4)+#9+a);
   if list_hhh then writeln(hhh,'#define '+name+'_'+a+' 0x'+Hex(ad,4));
  end;
end;


procedure save_str(var a:string; ofs:word);
var i:integer;
begin
 i:=High(t_str);

 t_str[i].nr:=struct_nr;
 t_str[i].lab:=a;
 t_str[i].ofs:=ofs;

 SetLength(t_str,i+2);
end;


procedure s_lab(var a:string; ad:cardinal; ba:word; var old:string; symbol:char);
var tmp: cardinal;
    x: integer;
begin
// sprawdz czy nie ma juz takiej etykiety
// bo jesli jest to nie dopisuj nowej pozycji tylko popraw stara
 x:=l_lab(a);

 if struct then begin
  ad:=ad-struct_adres;
  ba:=0;
 end;

 if x<0 then begin
  x:=High(t_lab); SetLength(t_lab,x+2);
 end else
  if t_lab[x].bnk<>ba then blad(old,2);

 tmp:=fCRC(a);
 if symbol<>'?' then
  if (t_lab[x].pas=pass_end) and (t_lab[x].nam=tmp) then blad(old,2);

 t_lab[x].nam:=tmp;
 t_lab[x].adr:=ad;
 t_lab[x].bnk:=ba;
 t_lab[x].pas:=pass;
 t_lab[x].blk:=blok;

 if blok>0 then t_lab[x].rel:=true;

 if pass=0 then
  if struct then save_str(a,ad);

 zapisz_etykiete(a,ad,ba,symbol);
end;


procedure save_lab(var a:string; ad:cardinal; ba:word; var old:string);
// zapamietuje etykiete, adres, bank
var sym: char;
begin

 if a<>'' then begin

  sym:=a[1];

  if macro then a:=lokal_name+a+'::'+IntToStr(dek_macro_nr) else
   if run_macro then a:=macro_nr+lokal_name+a else
    if proc then a:=proc_name+lokal_name+a else
     a:=lokal_name+a;

  s_lab(a,ad,ba,old,sym);

 end;

end;


procedure save_mac(var a:string);
var i: integer;
begin
 i:=High(t_mac);

 t_mac[i]:=a;

 SetLength(t_mac,i+2);
end;


procedure save_smb(var a:string);
var i: integer;
begin
 i:=High(t_smb);

 t_smb[i].smb:=a;

 SetLength(t_smb,i+2);
end;


procedure save_rel(var a:integer; idx:integer; b:word);
var i: integer;
begin
 i:=High(t_rel);

 if vector then t_rel[i].adr:=a-rel_ofs else t_rel[i].adr:=a-rel_ofs+1;

 t_rel[i].blk:=b;
 if not(empty) then t_rel[i].idx:=idx else t_rel[i].idx:=-100;
 t_rel[i].blo:=blok;

 SetLength(t_rel,i+2);
end;


procedure save_wyw(var a,b:string; var c:integer);
var i:integer;
begin
 i:=High(t_wyw);

 t_wyw[i].zm:=a;
 t_wyw[i].pl:=b;
 t_wyw[i].nr:=c;

 SetLength(t_wyw,i+2);
end;


function wartosc(var a:string; var v:Int64; o:char): cardinal;
var i: Int64;
    x: Boolean;
begin
 Result:=v;

 i:=abs(v);
 
 x:=false;

 case o of
  'B'            : x := i>$FF;
  'A','V'        : x := i>$FFFF;
  'E','H','L','T': x := i>$FFFFFF;
  'F','G'        : x := i>$FFFFFFFF;
 end;

 if x then blad(a,0);
end;


procedure formatuj(var a:string);
//
// zamienia tabulatory na spacje (1tab=8spc)
// jesli w linii sa tylko spacje zamienia ja na pusta linie
//
var s, i, j, k, len: word;
    Result: string;
begin
 Result:='';

 // zastepujemy tabulatory spacjami
 i:=1; k:=1; s:=0;

 while k<=length(a) do begin
  case a[k] of
   #9: begin
        len:=i mod 8; if len>0 then len:=8-len;
        for j:=0 to len do Result:=Result+' ';
        inc(i,len);
       end;
  ' ': begin inc(s); Result:=Result+' ' end;
  else
   Result:=Result+a[k];
  end;

  inc(i); inc(k);
 end;

 // sprawdzamy czy linia sklada sie z samych spacji
 if s=i-1 then Result:='';

 a:=Result;
end;


function BinToInt(var Value: String): cardinal;
var i, len: word;
begin
 Result:=0;
//remove leading zeros
 while Copy(Value,1,1)='0' do
  Value:=Copy(Value,2,Length(Value)-1);
//do the conversion
 len:=Length(Value);
 for i:=len downto 1 do
  if Copy(Value,i,1)='1' then
   Result:=Result+(1 shl (len-i));
end;


function _dec(var a:char): Boolean;
// dopuszczalne znaki dla liczb decymalnych
begin
 _dec := (a in ['0'..'9'])
end;

function _lab(var a:char): Boolean;
// dopuszczalne znaki dla etykiet
begin
 _lab := (UpCase(a) in ['A'..'Z','0'..'9','_','?','@','.'])
end;

function _bin(var a:char): Boolean;
// dopuszczalne znaki dla liczb binarnych
begin
 _bin := (a in ['0'..'1'])
end;

function _hex(var a:char): Boolean;
// dopuszczalne znaki dla liczb heksadecymalnych
begin
 _hex := (UpCase(a) in ['0'..'9','A'..'F'])
end;


function get_lab(var i:word; var a:string):string;
// pobierz etykiete zaczynajaca sie znakami 'A'..'Z','_','?','@'
begin
 Result:='';
 if UpCase(a[i]) in ['A'..'Z','_','?','@'] then
  while _lab(a[i]) do begin Result:=Result+UpCas_(a[i]); inc(i) end;
end;


function get_lab_2(var i:word; var a:string):string;
// pobierz niepusta etykiete zaczynajaca sie znakami 'A'..'Z','_','?','@'
begin
 omin_spacje(i,a); Result:=get_lab(i,a);

 if Result='' then blad(a,15);
end;


function get_string(var i:word; var a,old:string; test:Boolean):string;
// pobiera ciag znakow, ograniczony znakami '' lub ""
// podwojny '' oznacza literalne '
// podwojny "" oznacza literalne "
// TEST = TRUE sprawdza czy ciag jest pusty
var znak, gchr: char;
begin
Result:=''; gchr:=a[i];

 while i<=length(a) do begin
  inc(i);         // omijamy pierwszy znak ' lub "

  znak:=a[i];

  if znak=gchr then begin
   inc(i);
   if a[i]=gchr then znak:=gchr else begin
    if test then if Result='' then blad(old,3);
    exit;
   end;
  end;

  Result:=Result+znak;
 end;

 blad(a,3);       // nie napotkal znaku konczacego ciag ' lub "
end;


procedure test_nawias(var a:string; nawias:char; v:byte);
begin
 case nawias of
  '[': blad(a,6);
  '(': blad(a,7);
  '{': blad(a,19+v);
 end;
end;


function ciag_ograniczony(var i:word; var a:string; lewa,prawa:Char; zag:Boolean):string;
// pobiera ciag ograniczony dwoma znakami 'LEWA' i 'PRAWA'
// 'ZAG' okresla czy znaki 'LEWA' i 'PRAWA' moga byc zagniezdzone
var nawias: word;
    znak: char;
    petla: Boolean;
begin
 Result:='';
 nawias:=0; petla:=true;

 while petla and (i<=length(a)) do begin

  if a[i]=lewa then inc(nawias) else
   if a[i]=prawa then dec(nawias);

//  if lewa<>'{' then
//   if a[i]=' ' then blad(a,8);

//   if (lewa='(') or (lewa='[') then inc(nawias);

  if not(zag) then
   if nawias>1 then test_nawias(a,lewa,0);

  if nawias=0 then petla:=false;

  if a[i] in ['''','"'] then begin
   znak:=a[i];
   Result:=Result+znak+get_string(i,a,a,false)+znak
  end else begin
   Result:=Result+a[i];
   inc(i)
  end;

 end;

 if petla then test_nawias(a,lewa,1);

 Result:=copy(Result,2,length(Result)-2);
end;


function NewFileExt(var a:string; n:byte):string;
var i: word;
    b: string;
const
    ex: array [0..4] of string[3]=
    ('lst','obx','lab','mac','h');

begin
 b:='.'+ex[n];
 Result:=a+b;

 if a<>'' then begin
  i:=length(a);
  while (a[i]<>'.') and (i>=1) do dec(i);

  Result:=copy(a,1,i-1)+b;
 end;
end;


function get_datUp(var i:word; var a:string):string;
// pobiera ciag znakow, zmienia wielkosc liter na duze
begin
 Result:='';

 if a<>'' then
  while not(a[i] in [#0,';',' ']) and ((a[i]<>'/') and (a[i+1]<>'/')) do begin
   Result:=Result+UpCase(a[i]);
   inc(i);
  end;

end;


function get_ciag(var i:word; var zm:string):string;
// pobiera niepusty ciag znakow
begin
 omin_spacje(i,zm);

 Result:=get_datUp(i,zm);
 if Result='' then blad(zm,23);
end;


function get_dat(var i:word; var a:string; sep:Char; spacja:Boolean):string;
// wczytaj dowolne znaki, oprocz ' ' i ','
// jesli wystepuja znaki otwierajace ciag, czyta caly ciag
var znak: char;
    len: word;
begin
 Result:=''; len:=length(a);

 while i<=len do begin

  case UpCase(a[i]) of
   '(': Result:=Result+'('+ciag_ograniczony(i,a,'(',')',false)+')';
   '{': Result:=Result+'{'+ciag_ograniczony(i,a,'{','}',false)+'}';
   '[': Result:=Result+'['+ciag_ograniczony(i,a,'[',']',true)+']';

   'A'..'Z': while _lab(a[i]) do begin Result:=Result+UpCas_(a[i]); inc(i) end;

   '''','"':
     begin
      znak:=a[i];
      Result:=Result+znak+get_string(i,a,a,false)+znak;
     end;

   '/': if a[i+1]='/' then exit else begin Result:=Result+'/'; inc(i) end;

   ';': exit;

   ' ': if spacja then exit else begin Result:=Result+' '; inc(i) end;

  else
   begin
    if a[i]=sep then exit;
    Result:=Result+UpCase(a[i]); inc(i);
   end;
  end;

 end;

end;


procedure extra_char(var a:string; b:Boolean);
// jesli 'b=TRUE' to nastapila liczba po liczbie, a wiec
// zabraklo rozdzielajacego operatora arytmetycznego
begin
 if b then blad(a,4);
end;


function get_dat_noSPC(var i:word; var a:string; sep:Char; var old:string):string; 
begin
 Result:='';

 omin_spacje(i,a);

 while i<=length(a) do begin

  case a[i] of
   ' ': inc(i);

   '.','$','%':
        begin
         extra_char(old,(a[i+1] in [#0,' ',';','*']) or ((a[i+1]='/') and (a[i+2]='/')));
         Result:=Result+a[i]; inc(i);
        end;

   '/': if a[i+1]='/' then exit else begin Result:=Result+'/'; inc(i) end;

   '(': if white_spaces then begin
         Result:=Result+'('; inc(i)
        end else exit;

   ';': exit;

  else
   begin
    if sep<>' ' then
     if a[i]=sep then exit;

    Result:=Result+get_dat(i,a,sep,true);
   end;
  end;

 end;

end;


function get_value(var i:word; var a:string; d:char; var old:string): Int64;
// pobiera ciag znakow wg 'D' i zamienia na wartosc liczbowa
var tmp:string;
    err:integer;
begin
 tmp:='';

 case d of
  'H': begin
        while _hex(a[i]) do begin tmp:=tmp+a[i]; inc(i) end;
        tmp:='$'+tmp;
       end;

  'B': begin
        while _bin(a[i]) do begin tmp:=tmp+a[i]; inc(i) end;
        if tmp='' then extra_char(old,true);
        Result:=BinToInt(tmp); wartosc(old,Result,'F');
        exit
       end;

  'L','E': begin
        tmp:=get_lab(i,a);
        if d='L' then err:=load_lab(tmp,true) else err:=load_lab(tmp,false);

        if err<0 then
         if pass=pass_end then begin lokal_name:=tmp; blad(old,5) end;

        Result:=err; exit
       end;

  'D': while _dec(a[i]) do begin tmp:=tmp+a[i]; inc(i) end;
 end;

// zamieniamy tekst na wartosc liczbowa
// jesli sa jakies znaczki dziwaczki wystapi blad
 val(tmp,Result,err); if err<>0 then extra_char(old,true);

 wartosc(old,Result,'F');
end;


function OperExt(var i:word; var old:string; a,b:char; var value:Boolean):char;
// operatory zlozone zamieni na odpowiedni jednoliterowy kod
begin
 Result:=' ';
 if not(value) then blad_ill(old,a,b);

 case (ord(a) shl 8+ord(b)) of
  15422,8509: Result := 'A';       // '<>', '!='
       15421: Result := 'B';       // '<='
       15933: Result := 'C';       // '>='
       15420: Result := 'D';       // '<<'
       15934: Result := 'E';       // '>>'
       15677: Result := '=';       // '=='
        9766: Result := 'F';       // '&&'
       31868: Result := 'G';       // '||'
 else
  blad_ill(old,a,b);
 end;

{
 if (a='<>') or (a='!=') then Result:='A' else
  if a='<=' then Result:='B' else
   if a='>=' then Result:='C' else
    if a='<<' then Result:='D' else
     if a='>>' then Result:='E' else
      if a='==' then Result:='=' else
       if a='&&' then Result:='F' else
        if a='||' then Result:='G' else
         blad_ill(old,a,8);
}

 inc(i,2); value:=false
end;


function OperNew(var i:word; var old:string; a:char; var value:Boolean; b:Boolean): char;
begin
 if value<>b then blad_ill(old,a,#0);

 Result:=a;

 inc(i); value:=false;
end;


function oblicz_wartosc(a:string; var old: string): Int64;
// oblicza wartosc wyrazenia, uwzglednia operacje arytmetyczne
type znak_wart = record
                  znak: char;
                  wart: Int64;
                 end;

var i, j, k, len, b: word;
    tmp, txt: string;
    war, arg: Int64;
    petla, value: Boolean;
    oper: char;
    cash: array [0..16] of char;
    stos, stos_: array of znak_wart;
    op: array of char;

const
// piorytet operatorow
  prior: array [0..16] of char=
  ('D','E','&','|','^','*','/','%','+',{'-',}'=','A',
   'B','<','C','>','F','G');

begin
 if a='' then blad(old,23);
 Result:=0;

 // init tablicy 'OP'
 SetLength(op,2); op[0]:='+'; oper:='+';

 i:=1; war:=0; b:=0; len:=length(a);
 fillchar(cash,sizeof(cash),' ');               // wypelniamy spacjami

 value:=false;
 j:=1;               // na pozycji zerowej (j=0) tablicy 'STOS' dopiszemy '+0'
 SetLength(stos,3);

 while i<=len do begin

  case UpCase(a[i]) of

   // odczytaj operator '<'
   '<': if a[i+1] in ['<','=','>'] then
         oper:=OperExt(i,old,'<',a[i+1],value) else
          if value then oper:=OperNew(i,old,'<',value,true) else
           oper:=OperNew(i,old,'M',value,false);

   // odczytaj operator '>'
   '>': if a[i+1] in ['=','>'] then
         oper:=OperExt(i,old,'>',a[i+1],value) else
          if value then oper:=OperNew(i,old,'>',value,true) else
           oper:=OperNew(i,old,'S',value,false);

   // odczytaj operator '='
   '=': if a[i+1]='=' then
         oper:=OperExt(i,old,'=','=',value) else
          if value then oper:=OperNew(i,old,'=',value,true) else
           oper:=OperNew(i,old,'X',value,false);

   // odczytaj operator '&'
   '&': if a[i+1]='&' then
         oper:=OperExt(i,old,'&','&',value) else
          oper:=OperNew(i,old,'&',value,true);

   // odczytaj operator '|'
   '|': if a[i+1]='|' then
         oper:=OperExt(i,old,'|','|',value) else
          oper:=OperNew(i,old,'|',value,true);

   // odczytaj operator '!'
   '!': if a[i+1]='=' then
         oper:=OperExt(i,old,'!','=',value) else
          oper:=OperNew(i,old,'!',value,false);

   // odczytaj operator '^'
   '^': if not(value) then begin
         oper:='H'; inc(i)
        end else oper:=OperNew(i,old,'^',value,true);

   // odczytaj operator '/'
   '/': oper:=OperNew(i,old,'/',value,true);

   // odczytaj operator '*'
   '*': if not(value) then begin
         war:=adres; inc(i);
         value:=true;
        end else oper:=OperNew(i,old,'*',value,true);

   // odczytaj operator '~'
   '~': oper:=OperNew(i,old,'~',value,false);

   // odczytaj operatory '+' '-'
   '+','-':
        oper:=OperNew(i,old,a[i],value,value);

   // odczytaj wartosc decymalna
   '0'..'9':
        begin
         extra_char(old,value);
         war:=get_value(i,a,'D',old);
         value:=true;
        end;

   // odczytaj .OR, .AND, .XOR, .NOT, .DEF
   '.': begin
         IORes:=ord(a[i+1]) shl 16+ord(a[i+2]) shl 8;

         case IORes of
          $4F5200: oper:=OperExt(i,old,'|','|',value);  // OR

          $414E00:
               if a[i+3]='D' then begin
                oper:=OperExt(i,old,'&','&',value);     // AND
                inc(i,1);
               end;

          $584F00:
               if a[i+3]='R' then begin
                oper:=OperNew(i,old,'^',value,true);    // XOR
                inc(i,2);
               end;

          $4E4F00:
               if a[i+3]='T' then begin
                oper:=OperNew(i,old,'!',value,false);   // NOT
                inc(i,2);
               end;

          $444500:
               if a[i+3]='F' then begin                 // DEF
                inc(i,4);

                txt:=get_lab(i,a);
                arg:=l_lab(txt);
                war:=ord(arg>=0);

                value:=true;
               end;

         else
          blad(old,12);
         end;

         inc(i);
        end;

   // odczytaj etykiete i okresl jej wartosc
   // znak ':' oznacza etykiete globalna lub parametr makra
   'A'..'Z','_','?','@',':':
        begin
         petla:=false; b:=0; war:=0;

         if a[i]=':' then
          case UpCase(a[i+1]) of
           'A'..'Z','_','?','@': begin petla:=true; inc(i) end;
           '0'..'9': if macro or run_macro then begin
                      value:=false; oper:=' '; i:=len+1
                     end else blad_ill(old,':',#0);
          else
           blad(old,4);
          end;

         if i<=len then begin

          extra_char(old,value);
          if petla then arg:=get_value(i,a,'E',old) else
           arg:=get_value(i,a,'L',old);

          if arg>=0 then
           case t_lab[arg].bnk of
        // zamiana struktur na dane DTA
          __id_struct: begin
                        if not(dta_used) then blad(old,58); //war:=t_str[t_lab[arg].adr].ofs;

                        if a[i]='[' then begin
                         tmp:=ciag_ograniczony(i,a,'[',']',true);
                         k:=1; tmp:=get_dat_noSPC(k,tmp,' ',old);
                         war:=oblicz_wartosc(tmp,old);

                         writeln(war);
                        end;

      omin_spacje(i,a);
      if a[i]='{' then begin
       tmp:=ciag_ograniczony(i,a,'{','}',false);
       k:=1; tmp:=get_dat_noSPC(k,tmp,' ',old);

       k:=1; omin_spacje(k,tmp); //SetLength(par,1);
       while tmp[k]<>'}' do begin

        txt:=get_dat_noSPC(k,tmp,',',old);
        war:=oblicz_wartosc(txt,old);

        writeln(war);

//        _odd:=High(par); par[_odd]:=txt;
//        SetLength(par,_odd+2);

        omin_spacje(k,tmp);
        if tmp[k]=',' then inc(k) else Break;
        omin_spacje(k,tmp);
       end;

                        end;

//                        writeln(a,' , ',a[i],' , ',war);
                       end;

            __id_proc: begin
                        b:=StrToInt(t_mac[t_lab[arg].adr+2]);
                        war:=StrToInt(t_mac[t_lab[arg].adr+3]);
                       end;

             __id_smb: if not(branch) then begin
                        save_rel(adres,t_lab[arg].adr,0);
                        t_smb[t_lab[arg].adr].use:=true;
                        reloc:=true;
                        war:=__rel;
                       end;
           else

            begin
             if not(branch) then
              if t_lab[arg].rel then begin
               save_rel(adres,-1,t_lab[arg].blk);
               reloc:=true;
              end;

             b:=t_lab[arg].bnk;
             war:=t_lab[arg].adr;
            end;

           end;

          value:=true;
         end;

        end;

   // odczytaj wartosc hexadecymalna
   '$': begin
         extra_char(old,value);
         inc(i);     // omin pierwszy znak '$'
         war:=get_value(i,a,'H',old);
         value:=true;
        end;

   // odczytaj wartosc binarna lub potraktuj % jako operator
   // lub jako numer parametru dla makra gdy wystapia %%
   '%': if a[i+1]='%' then begin
         b:=0; war:=0;
         value:=false; oper:=' '; i:=len+1
        end else
         if value then oper:=OperNew(i,old,'%',value,true) else
          begin
           inc(i);     // omin pierwszy znak '%'
           war:=get_value(i,a,'B',old);
           value:=true;
          end;

   // odczytaj string ograniczony apostrofami ''
   '''','"':
        begin
         extra_char(old,value);
         k:=ord(a[i]);
         tmp:=get_string(i,a,old,true);
         if chr(k)='''' then war:=ord(tmp[1]) else war:=ata2int(ord(tmp[1]));

         if a[i]='*' then
          if a[i+1] in [#0,',',' ','&','|','^','*','/','%','+','-','=','<','>'] then begin
           war:=war+$80; inc(i)
          end;

         value:=true;
        end;

   // odczytaj wartosc miedzy { }
   '{': begin
         extra_char(old,value);

         tmp:=ciag_ograniczony(i,a,'{','}',false);  // test na poprawnosc
         k:=1; war:=oblicz_mnemonik__(k,tmp,old);
         war:=(war and $ff00) shr 8;

         value:=true;
        end;

   // odczytaj wartosc miedzy [ ]
   '[': begin
         extra_char(old,value);
         tmp:=ciag_ograniczony(i,a,'[',']',true);
         k:=1; tmp:=get_dat_noSPC(k,tmp,' ',old);
         war:=oblicz_wartosc(tmp,old);
         value:=true;
        end;

   // jesli napotkasz spacje to zakoncz przetwarzanie
   // do 'value' wpisz 'false' aby nie traktowal tego jako liczby
   ' ': begin
         value:=false; oper:=' '; i:=len+1
        end;

  else
   blad_ill(old,a[i],#0);
  end;

// jesli przetwarzamy wartosc numeryczna to zapisz na stosie
// t wartosc i operator
  if value then begin

   op[High(op)]:='+';

  // przetworz operatory z 'OP' cofajac sie
   for k:=High(op) downto 1 do
    case op[k] of
     '-': war := -war;
     'M': war := lo(war);
     'S': war := hi(war);
     'H': war := (war shr 16) and $FF;
     '!': war := ord(war=0);
     '~': war := not(war);
     'X': war := b;
    end;

   if op[0]='-' then begin war:=-war; op[0]:='+' end;

   stos[j].znak:=op[0];
   stos[j].wart:=war;

   cash[pos(op[0],prior)-1]:=op[0];

   // init tablicy 'OP'
   SetLength(op,1); oper:=' ';

   inc(j); if j>High(stos) then SetLength(stos,j+1);

  end else begin

   k:=High(op);
   op[k]:=oper;
   SetLength(op,k+2);

  end;

 end;

 if oper<>' ' then blad(old,23);

// obliczenie wartosci wg piorytetu obliczen
// na koncu dodawaj tak dlugo az nie uzyskasz wyniku
 len:=0; war:=0;

 while cash[len]=' ' do inc(len);
 if len>=sizeof(cash) then exit;      // nie wystapil zaden operator

 stos[0].znak:='+'; stos[0].wart:=0;
 stos[j].znak:='+'; stos[j].wart:=0;
 inc(j);
 SetLength(stos,j+1);
 SetLength(stos_,j+1);

 while j>1 do begin
 // 'petla=true' jesli nie wystapi zaden operator
 // 'petla=false' jesli wystapi operator
  i:=0; petla:=true;

  while i<=j-2 do begin
    war:=stos[i].wart;
    oper:=stos[i+1].znak;
    arg:=stos[i+1].wart;

 // obliczaj wg kolejnosci operatorow z 'prior'
     if oper=cash[len] then begin
        case oper of
         '+': war := war + arg;
         '&': war := war and arg;
         '|': war := war or arg;
         '^': war := war xor arg;
         '*': war := war * arg;
         '/': if arg=0 then blad(old,26) else war := war div arg;
         '%': war := war mod arg;
         '=': war := ord(war=arg);
         'A': war := ord(war<>arg);
         'B': war := ord(war<=arg);
         '<': war := ord(war<arg);
         'C': war := ord(war>=arg);
         '>': war := ord(war>arg);
         'D': war := war shl arg;
         'E': war := war shr arg;
         'F': war := ord(war<>0) and ord(arg<>0);
         'G': war := ord(war<>0) or ord(arg<>0);
        end;

 // obliczyl nowa wartosc, skasuj znak
      stos[i].wart:=war; stos[i+1].znak:=' ';
      inc(i); petla:=false;
     end;

  inc(i)
  end;

// przepisz elementy ktore maja niepusty znak ze 'stos' do 'stos_'
  k:=0;
  for i:=0 to j-1 do
   if stos[i].znak<>' ' then begin stos_[k]:=stos[i]; inc(k) end;
  j:=k;

// przepisz zawartosc 'stos_' do 'stos' i dzialaj dalej
  stos:=copy(stos_,0,j);

  if petla and (len<sizeof(prior)-1) then begin
   inc(len);
   while cash[len]=' ' do inc(len);
  end;

 end;

Result:=war;
end;


procedure put_dst(a: byte);
begin
 if not(struct) then begin
  t_buf[buf_i]:=a; inc(buf_i);

  if buf_i>=sizeof(t_buf) then flush_dst;
 end; 
end;


procedure save_dst(a:byte);
var x, y, ex, ey: word;
    znk: char;
begin
 if (pass=pass_end) and (opt and 2>0) and not(macro) then begin

  if org then begin
   t:=copy(t,1,7);
   if hea and (opt and 1>0) then t:=t+'FFFF> ';

   x:=adres;
   y:=t_hea[hea_i];

   if hea_ofs>=0 then begin
    y:=hea_ofs+(y-x); x:=hea_ofs;
   end;

   if x<=y then begin
    ex:=x; ey:=y
   end else begin
    ey:=x; ex:=y
   end;

   znk:='-';
   if blok>0 then begin       // dlugosc bloku relokowalnego
    y:=y-x+1; znk:=','
   end;

   if adres>=0 then begin
    t:=t+Hex(bank,2)+',';
    if ex>ey then t:=t+Hex(t_hea[hea_i-1]+1,4) else t:=t+Hex(x,4);

    if (ex<=ey) and (opt and 1>0) then t:=t+znk+Hex(y,4)+'>';
   end;

   if hea and (opt and 1>0) then begin
    put_dst($ff); put_dst($ff);
   end;

   if opt and 1>0 then begin
    put_dst(lo(x)); put_dst(hi(x));
    put_dst(lo(y)); put_dst(hi(y));
   end;

   org:=false; hea:=false;
  end;

   if length(t)<31 then
    if length(t)+3>29 then begin t:=t+' +'; justuj end else t:=t+' '+Hex(a,2);

  put_dst(a);
 end;

end;


procedure save_dta(var war:Int64; var tmp:string; op_:char; invers:byte);
var k: word;
    v: byte;
begin

   case op_ of
    'L','B': begin save_dst(lo(war)); inc(adres) end;

    'H': begin save_dst(hi(war)); inc(adres) end;

    'A','V': begin save_dst(lo(war)); save_dst(hi(war)); inc(adres,2) end;

    'T','E':
         begin
          save_dst(war and $ff); save_dst((war shr 8) and $ff);
          save_dst((war shr 16) and $ff);
          inc(adres,3)
         end;

    'F': begin
          save_dst(war and $ff); save_dst((war shr 8) and $ff);
          save_dst((war shr 16) and $ff); save_dst((war shr 24) and $ff);
          inc(adres,4)
         end;

    'G': begin
          save_dst((war shr 24) and $ff); save_dst((war shr 16) and $ff);
          save_dst((war shr 8) and $ff); save_dst(war and $ff);
          inc(adres,4)
         end;

    'C': for k:=1 to length(tmp) do begin
          v:=ord(tmp[k])+invers;
          save_dst(v); inc(adres)
         end;

    'D': for k:=1 to length(tmp) do begin
          v:=ata2int(ord(tmp[k])+invers);
          save_dst(v); inc(adres)
         end;
   end;
end;


function get_expres(var i:word; var a,old:string):Int64;
// pobierz ciag znakow ograniczony przecinkiem i oblicz jego wartosc
// nie wczytuj nawiasow zamykajacych ')'
var tmp: string;
    k: word;
begin
 k:=i;
 tmp:=get_dat(k,a,')',false);
 tmp:=copy(a,1,k-1);
 tmp:=get_dat_noSPC(i,tmp,',',old);

 Result:=oblicz_wartosc(tmp,old);
end;


function get_func(var i:word; var a,old:string; b:Boolean):integer;
begin
 Result:=get_expres(i,a,old);

 if b then if a[i]<>',' then blad(old,52) else inc(i);
end;


function test_string(var i:word; var a:string):byte;
var war: Int64;
    txt: string;
begin
 Result:=0;

 case a[i] of
      '*': begin Result:=128; inc(i) end;
  '+','-': begin
            txt:=get_dat(i,a,',',true);
            war:=oblicz_wartosc(txt,a); wartosc(a,war,'B');
            Result:=war;
           end;
 end;

end;


procedure oblicz_dane(var i:word; var a,old:string; typ: byte);
// zwraca wartosci liczbowe ciagu DTA, .BYTE, .WORD, .LONG, .DWORD
var k: word;
    op_, default: char;
    sin_a, sin_b, sin_c, sin_d, sin_e, x: integer;
    war: Int64;
    step, val: real;
    value, nawias, ciag: Boolean;
    tmp: string;
    invers: byte;
begin
 // pola struktury musza posiadac etykiety
 if struct then
  if not(label_used) then blad(old,15);

 war:=0; invers:=0;

 value:=false; nawias:=false; ciag:=false; branch:=false;

 tmp:=' ';

// B( -> BYTE, A( -> WORD, V( -> VECTOR (WORD)
// L( -> BYTE, H( -> BYTE
// T( -> 24bit, E( -> 24bit
// F( -> DWORD
// G( -> DWORD (odwrocona kolejnosc)
// C' -> ATASCII
// D' -> INTERNAL
// sin(centre,amp,size[,first,last])

 // okreslenie domyslnego typu wartosci
 op_ := 'B';       // dla 'typ'=1

 case typ of
  2: op_ := 'A';
  3: op_ := 'T';
  4: op_ := 'F';
 end;

 default := op_;

 // jesli brakuje danych to generuj domyslnie zera
 if a[i]=#0 then begin
  war:=0;
  save_dta(war,tmp,op_,0);
  exit;
 end;

 if struct then blad(old,58);

 vector:=true; dta_used:=true;

 while i<=length(a) do begin

// SIN, RND
  if UpCase(a[i]) in ['S','R'] then begin
   x:=ord(UpCase(a[i+1])) shl 16+ord(UpCase(a[i+2])) shl 8+ord(a[i+3]);

  CASE x of
 // IN(
   $494E28: begin
    inc(i,3);

   // sprawdzamy poprawnosc nawiasow
    k:=i; tmp:=ciag_ograniczony(k,a,'(',')',false);

    value:=true; invers:=0; tmp:='';

    inc(i);
    sin_a:=get_func(i,a,old,true);
    sin_b:=get_func(i,a,old,true);
    sin_c:=get_func(i,a,old,false);

    sin_d:=0;
    sin_e:=sin_c-1;

    if a[i]=',' then begin

     inc(i);
     sin_d:=get_func(i,a,old,true);
     sin_e:=get_func(i,a,old,false);

     inc(i);
    end else inc(i);

    step:=360/sin_c; val:=0;

    for x:=sin_d to sin_e do begin
     war:=round(sin(val*(pi/180))*sin_b)+sin_a;

     val:=val+step;

     war:=wartosc(old,war,op_);
     save_dta(war,tmp,op_,invers)
    end;

    omin_spacje(i,a);
   end;

 // ND(
   $4E4428: begin
    inc(i,3);

   // sprawdzamy poprawnosc nawiasow
    k:=i; tmp:=ciag_ograniczony(k,a,'(',')',false);

    value:=true; invers:=0; tmp:='';

    inc(i);
    sin_a:=get_func(i,a,old,true);
    sin_b:=get_func(i,a,old,true);
    sin_c:=get_func(i,a,old,false);
    inc(i);

    sin_d:=sin_b-sin_a;

    randomize;
    for x:=0 to sin_c-1 do begin

     war:=sin_a+random(sin_d+1);

     war:=wartosc(old,war,op_);
     save_dta(war,tmp,op_,invers)
    end;

    omin_spacje(i,a);
   end;

  END;    //end case

  end;

  case UpCase(a[i]) of
   'A','B','E','F','G','H','L','T','V':
            if a[i+1]='(' then begin
            // pobierz ciag ograniczony nawiasami ()
             extra_char(old,value);
             op_:=UpCase(a[i]); inc(i);
            // sprawdzamy poprawnosc
             k:=i; tmp:=ciag_ograniczony(k,a,'(',')',true);
             nawias:=true;

            // dane powyzej 2 bajtow nie sa relokowalne
             if op_ in ['E','F','G','T'] then branch:=true else branch:=false;

             inc(i);             // omin nawias otwierajacy

            end else begin
             extra_char(old,value);
             war:=get_expres(i,a,old);
             value:=true
            end;

   'C','D': if a[i+1] in ['''','"'] then begin
            // pobierz ciag ograniczony apostrofami '' lub ""
             op_:=UpCase(a[i]); inc(i);

            end else begin
             extra_char(old,value);
             war:=get_expres(i,a,old);
             value:=true
            end;

   '''','"': begin
              extra_char(old,value);

              tmp:=get_string(i,a,old,true);
              war:=0; ciag:=true;
              invers:=test_string(i,a);

              if not(op_ in ['C','D']) then op_:='C';

              if pisz then write(tmp);

              value:=true;
             end;

   '(': begin nawias:=true; inc(i) end;

//   blad(old,11);

   ')': if nawias and value then begin
         value:=false; nawias:=false; inc(i);
         omin_spacje(i,a);
        end else blad(old,4);

   ',': begin
         inc(i); invers:=0; value:=false;

         omin_spacje(i,a);

         if a[i] in [#0,' '] then blad(old,23);

         if not(nawias) then op_:=default;

//         writeln(a,' -> ',op_,' -> ',nawias);
//         halt;
        end;

   ' ': omin_spacje(i,a);

   '/': if a[i+1]='/' then begin
         value:=false; i:=length(a)+1
        end else extra_char(old,true);

   ';': begin value:=false; i:=length(a)+1 end;

  else
       begin
         if a[i] in [#0,' '] then Break;

         extra_char(old,value);
         war:=get_expres(i,a,old);
         value:=true;

         if pisz then write('$'+hex(war,8));
       end;
  end;


  if value then begin

   if not(ciag) then
    if op_ in ['C','D'] then op_:='B';

   if not(pisz) then begin
    war:=wartosc(old,war,op_);
    save_dta(war,tmp,op_,invers);
   end;

   ciag:=false;
  end;

  omin_spacje(i,a);

 end;

 vector:=false; dta_used:=false;
end;


function value_code(var a: Int64; var old: string; test:Boolean): char;
// przedstawia wartosc w postaci kodu literowego
begin
 Result:='T';

 case abs(a) of
           0..$FF: Result:='Z';
      $100..$FFFF: Result:='Q';
  $10000..$FFFFFF: if test then Result:='T' else blad(old,14);
 else
  if not(test) then blad(old,0);
 end;

end;


procedure put_lst(a:string);
begin
 if (pass=pass_end) and (a<>'') then begin
  if (opt and 4>0) then writeln(lst,a);
  if (opt and 8>0) then writeln(a);
  t:='';
 end;
end;


procedure save_lst(a:Int64; var b:string; c:char); overload;
// zapis do pliku LST oraz DST
var i, len: byte;
begin

 if not(loop_used) and (pass=pass_end) then begin
  str(line,t);
  while length(t)<6 do t:=' '+t;
  t:=t+' ';
 end;

 if pass=pass_end then
  case c of
 // jesli pusta linia to usun spacje, czyli oszczedzamy bajty :)
   ';': begin
         if b<>'' then justuj else delete(t,length(t),1);
         put_lst(t+b);
        end;

   'l': t:=t+'= '+Hex(bank,2)+','+Hex(a,4);

   'i': t:=t+Hex(bank,2)+','+Hex(adres,4);
  end;


  if c='a' then
     if not(macro) then begin
      len:=lo(a);

      if pass=pass_end then begin
       if not(hea) and not(loop_used) and (adres>0) then
        t:=t+Hex(bank,2)+','+Hex(adres,4);

       if len>0 then
        for i:=1 to len do save_dst(a shr (i shl 3));

      end;

      inc(adres,len);
     end;
end;


procedure zapisz_lst(var a: string);
begin
  justuj; put_lst(t+a);
  a:='';
end;


procedure wymus_zapis_lst(var a: string);
begin
 save_lst(0,a,' '); justuj; put_lst(t+a);
end;


procedure test_eol(i:word; var a,old:string; b:char);
begin
 extra_char(old,not(a[i] in [#0,';',' ',b]));
end;


function oblicz_tst(var a:string; i:word):cardinal;
begin
 Result:=ord(a[i]) shl 24+ord(a[i+1]) shl 16+ord(a[i+2]) shl 8+ord(a[i+3]);
end;


function oblicz_mnemonik(var i:word; var a,old:string): Int64;
// zwraca wartosc typu INT64
// bajt 0  -> liczba elementow 0..3
// bajt 1  -> kod maszynowy mnemonika
// bajt 2  -> mlodszy bajt argumentu (lo)
// bajt 3  -> starszy bajt argumentu (hi)

{
(ZX)  $01  0
Z     $02  4
#     $04  8
Q     $08  12
(Z)Y  $10  16
ZX    $20  20
ZY    $40  20
QY    $80  24
QX   $100  28
(Q)  $200  32
}

var len, j, m: word;
    op_, mnemo, mnemo_tmp, zm, tmp: string;
    war, war_roz: Int64;
    code, ile, k: byte;
    op, siz: char;
    test, zwieksz, incdec, mvnmvp: Boolean;
    tryb, tst: cardinal;
begin
 Result:=0; if a='' then blad(old,12);

 op_:=''; siz:=' '; op:=' ';

 war_roz:=0;

 mvnmvp:=true;

 zwieksz:=false; incdec:=false; reloc:=false; branch:=false;

 omin_spacje(i,a);

// pobierz nazwe rozkazu, oblicz jego CRC16
 mnemo:=''; mnemo_tmp:='';
 while not(a[i] in [#0,' ','#',';','(','[','{','+','-','''','"','*','=']) do begin
  mnemo:=mnemo+UpCase(a[i]);
  mnemo_tmp:=mnemo_tmp+UpCas_(a[i]);
  inc(i);
 end;

 if mnemo='' then exit;

 // wyjatek znak '=' jest rownowazny 'EQU'
 // jesli wystapi znak ':' lub '%%' to sprawdz czy to nie parametr dla makra
 // wyjatkowo jesli przypomina rokaz XASM'a zamien ':' na '?'
 j:=pos(':',mnemo);
 if j<=0 then begin
  j:=pos('%%',mnemo);
  if j>0 then inc(j)
 end;

 if mnemo='=' then begin Result:=$80; exit end else
  if j>0 then
   if mnemo[j+1] in ['0'..'9'] then exit else
    if (length(mnemo)=7) and (mnemo[4]=':') then begin
     mnemo[4]:='?'; mnemo_tmp[4]:='?';
     a[i-4]:='?';
    end else blad(a,12);

// sprawdz czy 'MNEMO' sklada sie z poprawnych znakow
// jesli zaczyna sie od innego to potraktuj jako komentarz
 if not(mnemo[1] in ['A'..'Z','_','?','@']) then exit;
 for j:=1 to length(mnemo) do
  if not(_lab(mnemo[j])) then blad(a,12);


// sprawdz czy to operacja przypisania '=' lub 'EQU'
 j:=i;
 omin_spacje(i,a);

 if a[i]<>'=' then tmp:=get_datUp(i,a) else
  begin tmp:='='; inc(i) end;

 if (tmp='=') or (tmp='EQU') then begin
  war:=l_lab(mnemo_tmp);

  if t_lab[war].bnk<__id_param then begin

   if not(macro) then begin
    if war<0 then s_lab(mnemo_tmp,0,bank,a,mnemo_tmp[1]);

    branch:=true;              // dla EQU nie ma relokowalnosci
    tmp:=get_dat_noSPC(i,a,',',a);
    war:=oblicz_wartosc(tmp,a);
    s_lab(mnemo_tmp,war,bank,a,mnemo_tmp[1]);

    save_lst(war,a,'l');
   end;

   Result:=__nill;
   exit
  end;
 end;

 i:=j;

// sprawdz czy wystapilo rozszerzenie mnemonika
 if a[i-2]='.' then
  if length(mnemo)=5 then
   if UpCase(a[i-1]) in ['A','B','T','W','Q','Z','L'] then
     case UpCase(a[i-1]) of
      'A','W','Q': siz:='Q';
          'B','Z': siz:='Z';
          'T','L': siz:='T';
     end;

// poszukaj mnemonika w tablicy HASH
 j:=0;
 if length(mnemo)=3 then j:=hash[fCRC16(mnemo)] else
  if siz<>' ' then begin
   tmp:=copy(mnemo,1,3);
   j:=hash[fCRC16(tmp)];
   if j>0 then mnemo:=tmp else j:=0; // blad(a,32)
  end;


// sprawdz czy to nazwa makra
 if j=0 then begin
  // poszukaj w etykietach
    war:=load_lab(mnemo_tmp,false);

  // nie ma takiego makra
    if (war<0) and (pass=pass_end) then begin
      lokal_name:=mnemo;
      blad(a,35)
     end else
      if war<0 then exit;

    tryb:=war;

    if (t_lab[tryb].bnk=__id_macro) and macro then begin Result:=__nill; exit end;

    if t_lab[tryb].bnk>=__id_macro then Result:=t_lab[tryb].adr shl 8+lo(t_lab[tryb].bnk) else
     begin
      lokal_name:=mnemo; blad(a,35)
     end;

    exit;
  end;

 dec(j);

// znalazl pseudo rozkaz
 if j>$7f then
  if siz<>' ' then blad(a,33) else begin Result:=j; exit end;

 if not(macro) then
  if first_org and (opt and 1>0) then blad(a,10);

 if empty then blad(old,12);      // w bloku EMPTY nie moga wystepowac rozkazy

 if (opt and 16>0) then begin
  code:=kod_16[j];
  tryb:=ads_16[j];
 end else begin
  if j>55 then blad(a,14);
  code:=kod[j];
  tryb:=ads[j];
 end;

// dla .STRUCT nie ma mozliwosci uzywania rozkazow CPU
 if struct then blad(a,58);

// jesli mnemonik nie wymaga argumentu to skoncz
 if tryb=0 then begin
  if siz<>' ' then blad(a,33);
  test_eol(i,a,a,#0);
  Result:=code shl 8+1; exit
 end;

// omin spacje i przejdz do argumentu
// if a[i]<>' ' then blad(a,13);

 omin_spacje(i,a);

 // pobierz operator argumentu do 'OP'
 // nie dotyczy to rozkazow MVN i MVP (65816)
// tst:=ord(mnemo[1]) shl 16+ord(mnemo[2]) shl 8+ord(mnemo[3]);
 tst:=oblicz_tst(mnemo,1) shr 8;

// if (mnemo<>'MVN') and (mnemo<>'MVP') then begin
 if (tst<>$4D564E) and (tst<>$4D5650) then begin
  mvnmvp:=false;
  if a[i] in ['#','<','>'] then begin
   branch:=true;                     // nie relokuj tego rozkazu
   op:=a[i]; inc(i)
  end else op:=' ';

  if (a[i]='@') and (a[i+1] in [#0,' ']) then begin op:='@'; inc(i) end;

  if (op='@') then
   if (siz<>' ') then blad(a,33) else test_eol(i,a,a,#0);
 end;

 omin_spacje(i,a);

 // wyjatki dla ktorych rozmiar rejestru jest staly
 if (opt and 16>0) and (op='#') then begin

 // if mnemo='PEA' then
  if tst=$504541 then
   if siz in [' ','Q'] then siz:='Q' else blad(a,14);

 // if (mnemo='COP') or (mnemo='REP') or (mnemo='SEP') then
  if (tst=$434F50) or (tst=$524550) or (tst=$534550) then
   if siz in [' ','Z'] then siz:='Z' else blad(a,14);

 end;

 omin_spacje(i,a);
 zm:=copy(a,i,length(a));

 if white_spaces then begin
  m:=1; zm:=get_dat_noSPC(m,zm,' ',a);
 end;

 tmp:='';

 // jesli brak argumentu przyjmij domyslnie aktualny adres
 // lub znak '@' dla ASL, ROL, LSR, ROR
  if (zm='') and (op=' ') then
   if (tst=$41534C) or (tst=$4C5352) or (tst=$524F4C) or (tst=$524F52) then op:='@' else zm:='*';

// jesli jest nawias otwierajacy '(' lub '[' to sprawdz poprawnosc
// i czy nie ma miedzy nimi spacji
 if zm<>'' then begin
  j:=1;

 if (op=' ') and (zm[1] in ['(','[']) then begin

 if white_spaces then tmp:=get_dat_noSPC(j,zm,',',a)
  else tmp:=get_dat(j,zm,',',true);

 if length(tmp)=2 then blad(a,14);  // nie bylo argumentu pomiedzy nawiasami

 // jesli po nawiasie wystapi znak inny niz ',',' ',#0 to BLAD
  if white_spaces then omin_spacje(j,zm);
  test_eol(j,zm,a,',');

 // przepisz znak nastepujacy po ',' do 'OP_'
 if zm[j]=',' then begin

  __inc(j,zm);

//  if ((zm[j+1]=':') and (zm[j+2] in ['0'..'9'])) or ((zm[j+1]='%') and (zm[j+2]='%') and (zm[j+3] in ['0'..'9']))  then begin Result:=__nill; exit end else
  if UpCase(zm[j+0]) in ['X','Y','S'] then begin
   op_:=op_+UpCase(zm[j+0]); //zm[j]:=' ';

    if zm[j+1] in ['+','-'] then begin
     extra_char(a,UpCase(zm[j+0])='S');   // + - nie moze byc dla 'S'
     incdec:=true;
      case zm[j+1] of
       '-': war_roz:=oblicz_wartosc('{de'+zm[j+0]+'}',a);
       '+': war_roz:=oblicz_wartosc('{in'+zm[j+0]+'}',a);
      end;

     test_eol(j+2,zm,a,#0);
    end else test_eol(j+1,zm,a,#0);

  end else blad(a,14);
 end;

 // usun z ciagu znaki '()' lub '[]' i przepisz do 'OP_'
  op_:=op_+zm[1];
  zm:=copy(tmp,2,length(tmp)-2);
  j:=1;
 end;

 if white_spaces then tmp:=get_dat_noSPC(j,zm,',',a)
  else tmp:=get_dat(j,zm,',',true);

 if white_spaces then omin_spacje(j,zm);

// teraz jesli wystapi znak inny niz ',' to BLAD
 case op of
  '<','>','#','@': test_eol(j,zm,a,#0);
 else
  test_eol(j,zm,a,',');
 end;


// przepisz znak nastepujacy po ',' do 'OP_', jesli jest to 'XYS'
// w przeciwnym wypadku wylicz wartosc po przecinku
 if zm[j]=',' then
  if mvnmvp then begin
//   inc(j);
   __inc(j,zm);
   war_roz:=oblicz_wartosc(get_dat(j,zm,' ',true),a);
   op_:=op_+value_code(war_roz,a,true);
  end else begin

  __inc(j,zm);

//   if ((zm[j+1]=':') and (zm[j+2] in ['0'..'9'])) or ((zm[j+1]='%') and (zm[j+2]='%') and (zm[j+3] in ['0'..'9']))  then begin Result:=__nill; exit end else
   if UpCase(zm[j+0]) in ['X','Y','S'] then begin
    op_:=op_+UpCase(zm[j+0]);

    if zm[j+1] in ['+','-'] then begin
     if zm[j+2] in [#0,' '] then begin
      extra_char(a,UpCase(zm[j+0])='S');   // + - nie moze byc dla 'S'
      incdec:=true;
       case zm[j+1] of
        '-': war_roz:=oblicz_wartosc('{de'+zm[j+0]+'}',a);
        '+': war_roz:=oblicz_wartosc('{in'+zm[j+0]+'}',a);
       end;

      inc(j,2);
     end else begin
      zwieksz:=true; inc(j,1);
      war_roz:=oblicz_wartosc(get_dat(j,zm,' ',true),a);
     end;

    end else inc(j,1);

   end else blad(a,14);
  end;

//  end;
  test_eol(j,zm,a,#0);
 end;


 // jesli to rozkaz skoku to BRANCH=TRUE
 if (mnemo[1]='B') and (mnemo[2]<>'I') then branch:=true;

// oblicz wartosc argumentu mnemonika, jesli brak argumentu to 'WAR=0'
 war:=0;
 if not((tmp='') and (op='#')) then
  if op<>'@' then war:=oblicz_wartosc(tmp,old); 

 if zwieksz then war:=war+war_roz;

 op_:=op_+value_code(war,a,true);

 if mvnmvp then war:=war+(war_roz and $ff) shl 8;


 // wyjatki dotycza skokow warunkowych B??
 if (mnemo[1]='B') and (mnemo[2]<>'I') then begin
  if siz<>' ' then blad(a,33);

  op_:='B';

   test:=false; war:=war-2-adres; war_roz:=war;

   if (war<0) and (abs(war)-128>0) then begin war:=abs(war)-128; test:=true end;
   if (war>0) and (war-127>0) then begin war:=war-127; test:=true end;

   if test then
    if (mnemo[2]='R') and (mnemo[3]='A') then begin    // mnemo='BRA'
     op_:='W'; test:=false; war:=war_roz-1;
     if (war<0) and (abs(war)-32768>0) then begin war:=abs(war)-32768; test:=true end;
     if (war>0) and (war-32767>0) then begin war:=war-32767; test:=true end;
    end;

   if (pass=pass_end) and test then blad(a,-war);
 end;


 // na podstawie 'OP' okresl adresacje 'OP_'
 case op of
  '<','>','#': op_:='#';
          '@': op_:='@';
 end;


// znajdz obliczona adresacje w tablicy 'adresacja'
// oraz sprawdz czy dla tego mnemonika jest mozliwa ta adresacja
//
// jesli nie znalazl to zmien rodzaj argumentu Z->Q

 // jesli 65816 to wstaw znak '~'
 if (opt and 16>0) then op_:='~'+op_;

 len:=length(op_);

// jesli rozkaz relokowalny wymus rozmiar 'Q'
 if reloc then
  if siz='T' then blad(a,12) else
   if not(op in ['<','>']) then siz:='Q';


// sprawdz czy wystapilo rozszerzenie mnemonika
// zmodyfikuj wielkosc operandu na podstawie 'SIZ'
 if siz<>' ' then
  case op_[len] of
   'Q': if siz<>'Z' then op_[len]:=siz else
         if pass=pass_end then blad(a,31);

   'T': if siz='T' then op_[len]:='T' else
         if pass=pass_end then blad(a,31);

   '#': if siz='T' then blad(a,14) else
         if (siz='Z') and (abs(war)>$FF) then blad(a,31);
   'Z': op_[len]:=siz;
  end;

 j:=hash[fCRC16(op_)];

 if (j=0) or (tryb and maska[j]=0) then
  if op_[len]='Z' then begin
   op_[len]:='Q';
   j:=hash[fCRC16(op_)];
   if (j=0) or (tryb and maska[j]=0) then blad(a,14);
  end else if pass=pass_end then blad(a,14);


 if opt and 16>0 then begin
  ile:=0;
  for k:=8 downto 1 do
   if ((tryb shr 24) and maska[k]>0) then ile:=(9-k)*23;

{  if tryb and $80000000>0 then ile:=23;
  if tryb and $40000000>0 then ile:=2*23;
  if tryb and $20000000>0 then ile:=3*23;
  if tryb and $10000000>0 then ile:=4*23;
  if tryb and $08000000>0 then ile:=5*23;
  if tryb and $04000000>0 then ile:=6*23;
  if tryb and $02000000>0 then ile:=7*23;
  if tryb and $01000000>0 then ile:=8*23;}

  code:=code+addycja_16[j+ile];

 end else
  if (tryb and $80000000>0) then
   code:=code+addycja[j+12] else code:=code+addycja[j];


// obliczenie wartosci 'WAR' na podstawie 'OP'
 case op of
  '<': war:=lo(wartosc(a,war,'L'));
  '>': war:=hi(wartosc(a,war,'H'));
 end;

// policz z ilu bajtow sklada sie rozkaz wraz z argumentem
 ile:=1;

 if not(mvnmvp) then
  if incdec then begin
   inc(ile);
   case op_[len] of
    'Z': war:=war+(war_roz and $ff) shl 8;
    'Q': war:=war+(war_roz and $ff) shl 16;
    'T': war:=war+(war_roz and $ff) shl 24;
   end;
  end;

 if op_='~ZZ' then inc(ile,2)
 else
  case op_[len] of
   '#': begin
         if siz=' ' then siz:=value_code(war,a,false);

         case siz of
          'Z': begin war:=wartosc(a,war,'B'); inc(ile) end;
          'Q': if (opt and 16=0) then blad(a,14) else
               begin
                war:=wartosc(a,war,'A'); inc(ile,2)
               end;
         end;
        end;
   'Z','B': inc(ile);
   'Q','W': inc(ile,2);
   'T': inc(ile,3);
  end;

 Result:=ile+code shl 8+(war and $ff) shl 16+((war shr 8) and $ff) shl 24+((war shr 16) and $ff) shl 32;
end;


function test_opt(a:char; v,b:byte):byte;
//
// sprawdz czy sa znaki '+','-'
//
begin
 if v and b>0 then dat:=true;

 case a of
  '+': opt:=opt or b;
  '-': opt:=opt and (b xor $FF);
 else
  dat:=true;
 end;

 Result:=v or b;
end;


procedure reserved_word(var ety,zm:string);
begin
 if length(ety)=3 then                       // nazwa zarezerwowana
  if hash[fCRC16(ety)]>0 then blad(zm,9);

 if rept then blad(zm,51);
 if macro or proc then blad(zm,17);
 if load_lab(ety,true)>=0 then blad(zm,2);   // nazwa w uzyciu

 if ifelse<>0 then blad(zm,1);
end;


procedure opt_h_minus(a:byte);
begin
 opt:=opt and a;

 t_hea[hea_i]:=adres-1;
 inc(hea_i);
 if hea_i>High(t_hea) then SetLength(t_hea,hea_i+1);
end;


procedure zapisz_lokal;
begin
 t_loc[lokal_nr]:=lokal_name;
 inc(lokal_nr);

 SetLength(t_loc,lokal_nr+1);
end;


procedure oddaj_lokal(var a: string);
begin
 if lokal_nr=0 then blad(a,28);
 dec(lokal_nr);
 lokal_name:=t_loc[lokal_nr];
end;


procedure add_blok(var a:string);
begin
 inc(blok);
 if blok>$FF then blad(a,45);
end;


procedure save_blk(kod:integer; var a:string; h:Boolean);
var k: word;
    x, y, i, j: integer;
    hea_fd, hea_fe, ok, tst: Boolean;
    txt: string;
begin
 txt:=a;

 for k:=blok downto 0 do begin

  hea_fd:=false; ok:=false;

// szukaj w glownym bloku

  for y:=0 to High(t_rel)-1 do
  if t_rel[y].idx=kod then
   if t_rel[y].blo=0 then
    if t_rel[y].blk=k then begin

     if not(hea_fd) then begin
      if h then begin
       save_lst(0,txt,'a');

       save_dst($fd); save_dst($ff);
       save_dst( K );
       save_dst(0); save_dst(0);

       zapisz_lst(txt);
      end;

      hea_fd:=true; ok:=true;
     end;

     save_lst(0,txt,'a');

     save_dst($fd);
     save_dst(lo(t_rel[y].adr));
     save_dst(hi(t_rel[y].adr));

     zapisz_lst(txt);

     t_rel[y].idx:=-100;      // wylacz z poszukiwan
    end;

// szukaj w blokach relokowalnych
  for x:=1 to blok do begin

  hea_fe:=false; j:=0; tst:=false;

  for y:=0 to High(t_rel)-1 do
  if t_rel[y].idx=kod then
   if t_rel[y].blo=x then
    if t_rel[y].blk=k then begin

     if not(tst) then begin
      save_lst(0,txt,'a');
      tst:=true;
     end;

     if not(hea_fe) then begin
      if not(hea_fd) and h then begin
       save_dst($fd); save_dst($ff);
       save_dst( K );
       save_dst(0); save_dst(0);

       hea_fd:=true;
      end;

      save_dst($fe);
      save_dst(lo(x));

      hea_fe:=true; ok:=true;
     end;

     j:=t_rel[y].adr-j;
     if j>=$fa then
      for i:=0 to (j div $fa)-1 do begin
       save_dst($ff);
       dec(j,$fa);
      end;

     save_dst(lo(j));
     j:=t_rel[y].adr;

     t_rel[y].idx:=-100;      // wylacz z poszukiwan
    end;

   if tst then zapisz_lst(txt);
  end;

  if ok then begin
   save_lst(0,txt,'a');

   save_dst($fc);

   zapisz_lst(txt);
  end;

 end;

end;


function get_smb(var i:word; var zm:string):string;
var txt: string;
    x: byte;
begin
 txt:='';

 omin_spacje(i,zm);
 if zm[i] in ['''','"'] then txt:=get_string(i,zm,zm,true) else blad(zm,23);

// za dluga nazwa etykiety
 if length(txt)>8 then blad(zm,44);

// wyrownaj do 8 znakow
 while length(txt)<8 do txt:=txt+' ';

 for x:=1 to 8 do txt[x]:=UpCase(txt[x]);

 Result:=txt;
end;


function macro_rept_if_test:Boolean;
begin
// Result:=false;

// if not(macro) and not(rept) and if_test then Result:=true;

 Result := (not(macro) and not(rept) and if_test)
end;


procedure analizuj_linie(var zm,old_str,a:string; var rept_nr,rept_ile,nr:integer; var end_file,wyjscie:Boolean);
var g: file;
    v: byte;
    len, i, j, k, old_ifelse: word;
    _odd, _doo, idx: integer;
    old_run_macro, old_else_used, old_elseif_used, old_icl_used: Boolean;
    ety, txt, str, old_macro_nr: string;
    war, mne: Int64;
    indeks, tst: cardinal;
    par: array of string;

begin
  len:=length(zm);

  mne:=0;

  i:=1;

  // odczytaj etykiete z lewej strony
  ety:=get_lab(i,zm);

  label_used:=ety<>'';
  omin_spacje(i,zm);

  // dopuszczalne znaki na poczatku linii
  if i=1 then
   if not(zm[1] in [#0,'.',':']) then blad(zm,13);

  // znak '=' moze zastepowac EQU, nie musi byc poprzedzony spacja
  if zm[i]='=' then begin
   mne:=$80; __inc(i,zm);
  end;

  // zapisz makro lub rept
   if macro then begin
    save_lst(0,zm,' ');
    if (pass=0) and not(loop_used) then save_mac(zm) else
   end else
    if rept then begin
     save_lst(0,zm,' ');
     save_mac(zm);
     mne:=__nill;
     ety:='';         // koniecznie ETY=''
    end;


   if ifelse>0 then save_lst(0,zm,' ');

  // znak okreslajacy liczbe powtorzen linii
   if not(macro) and (zm[i]=':') then begin
    save_lst(0,zm,'a');
    inc(i);
    loop_used:=true;

    save_lab(ety,adres,bank,zm);

    _doo:=oblicz_wartosc(get_dat(i,zm,',',true),zm);
    if _doo<0 then blad(zm,0);

    idx:=High(t_mac);

    txt:=copy(zm,i,len);
    save_mac(txt);

    old_icl_used := icl_used;
    icl_used     := true;

    rept_line:=line-1;

    for _odd:=0 to _doo-1 do
     analizuj_mem__(idx,idx+1,zm,a,old_str);

    icl_used := old_icl_used;
    loop_used:= false;

    rept_line:=0;

    SetLength(t_mac,idx+1);

    mne:=__nill;
    ety:='';
   end;


  // sprawdz czy to rozkaz z kropka
   if zm[i]='.' then begin
    txt:=get_datUp(i,zm);
    mne:=hash[fCRC16(txt)];

    if (mne<__macro) or (mne>__over) then blad(zm,12);
   end;


 // wymuszamy zamiane IFT ELS EIF ELI na .IF .ELSE .ENDIF .ELSEIF
   if zm[i+3] in [#0,' '] then begin
     j:=i;

     txt:=get_datUp(i,zm);
     _odd:=hash[fCRC16(txt)]; dec(_odd);

     if not(_odd in [$8e..$91]) then i:=j else
          case _odd of
          // IFT
           $8e: mne:=__if;
          // ELS
           $8f: mne:=__else;
          // EIF
           $90: mne:=__endif;
          // ELI
           $91: mne:=__elseif;
          end;
   end;

 // odczytaj mnemonik
  if (mne=0) and if_test then
   if (zm[i]<>'') then mne:=oblicz_mnemonik(i,zm,zm);



 // BLK (kody $D0...)
 if lo(mne)=$93 then
 if ety<>'' then blad(zm,38) else begin

  empty:=false;

  txt:=get_ciag(i,zm);

  case UpCase(txt[1]) of
  //BLK D[os] a
  'D': mne := $82;

  //BLK S[parta] a
  'S': mne := $d0;

  //BLK R[eloc] M[ain]|E[xtended]
  'R': mne := $d1;

  //BLK E[mpty] a M[ain]|E[xtended]
  'E': begin mne := $d2; empty:=true end;

  //BLK N[one] a
  'N': begin mne := $82; opt_h_minus($fe) end;

  //BLK U[pdate] S[ymbols]
  //BLK U[pdate] A[ddress]
  //BLK U[pdate] N[ew] a text
  'U':
    if pass=pass_end then begin

     txt:=get_ciag(i,zm);

     save_lst(0,zm,'a');

     _doo:=0;    // liczba adresow do relokacji
     _odd:=0;    // liczba symboli do relokacji

     for idx:=0 to High(t_rel)-1 do
      if t_rel[idx].idx>=0 then inc(_odd) else
       if t_rel[idx].idx=-1 then inc(_doo);

     case UpCase(txt[1]) of
     // A[ddress]
      'A': if _doo>0 then begin

            save_blk(-1,zm,true);

            bez_lst:=true;
            first_org:=true;
           end;

     // S[ymbol]
      'S':  if _odd>0 then begin

             txt:=zm; zapisz_lst(txt);

             for _doo:=0 to High(t_smb)-1 do
              if t_smb[_doo].use then begin

              save_lst(0,txt,'a');

            // a($fffb),c'SMB_NAME',a(blk_len)
              save_dst($fb); save_dst($ff);

              txt:=t_smb[_doo].smb;
              for k:=1 to 8 do save_dst(ord(txt[k]));
              save_dst(0); save_dst(0);

              zapisz_lst(txt);

              save_blk(_doo,txt,false);
             end;

             bez_lst:=true;
            end;

     // N[ew]
      'N': begin
            txt:=zm; zapisz_lst(txt); str:='';

            omin_spacje(i,zm);
            txt:=get_dat(i,zm,' ',true); if txt='' then blad(zm,23);

            war:=l_lab(txt); if war<0 then begin lokal_name:=txt; blad(zm,5) end;
            k:=t_lab[war].blk; if k=0 then blad(zm,53);

            save_lst(0,str,'a');

          // dta a($fffc),b(blk_num),a(smb_off)
          // dta c'SMB_NAME'
            save_dst($fc); save_dst($ff);
            save_dst(lo(k));
            save_dst(lo(t_lab[war].adr)); save_dst(hi(t_lab[war].adr));

            zapisz_lst(str);
            save_lst(0,str,'a');

            txt:=get_smb(i,zm);
            for k:=1 to 8 do save_dst(ord(txt[k]));

            zapisz_lst(txt);
            bez_lst:=true;
           end;

     else
      blad(zm,23);
     end;

    end;

  else
   blad(zm,0);
  end;

 end;


 // zamien .DS na ORG
 // wymus wykonanie ORG *+
  if lo(mne)=__ds then begin

   wymus_zapis_lst(zm);

   omin_spacje(i,zm);
   txt:='org *+'+get_dat_noSPC(i,zm,' ',zm);
   i:=1; mne:=oblicz_mnemonik(i,txt,zm);
   zm:=txt;

//   bez_lst:=true;
  end;


 // wykonaj .PROC
 // wymus wykonanie makra @CALL
  if lo(mne)=__proc_run then
   if macro_rept_if_test then begin

       save_lab(ety,adres,bank,zm);
       wymus_zapis_lst(zm);

       indeks:=mne shr 8;

   if StrToInt(t_mac[indeks+4])>0 then begin

     // sprawdz liczbe parametrow
       omin_spacje(i,zm); SetLength(par,1);
       while not(zm[i] in [#0,';','*']) and ((zm[i]<>'/') and (zm[i+1]<>'/')) do begin

        txt:=get_dat(i,zm,',',true);
         if txt<>'' then
          if (txt[1]='"') and (txt[length(txt)]='"') then txt:=copy(txt,2,length(txt)-2);

        _odd:=High(par); par[_odd]:=txt;
        SetLength(par,_odd+2);

        omin_spacje(i,zm);
        if zm[i]=',' then __inc(i,zm) else Break;
//        omin_spacje(i,zm);
       end;

       k:=High(par);
       if k<>StrToInt(t_mac[indeks+4]) then blad(zm,40);

     // sprawdz typ parametrow
       ety:='';
       if k>0 then
        for j:=0 to k-1 do begin
         txt:=par[j]; v:=0;

         if txt='@' then begin
          ety:=ety+'''@'''+','; war:=0;
          v:=ord('Z');
         end else
          if txt[1]='#' then begin
           ety:=ety+'''#'''+',';
           war:=oblicz_wartosc(copy(txt,2,length(txt)),zm);
           v:=ord(value_code(war,zm,true));
          end else begin
            ety:=ety+''' '''+',';
            war:=oblicz_wartosc(txt,zm);
           end;

       // typ parametru w deklaracji procedury (B,W,L)
         if v<>0 then
          case t_mac[indeks+j+5][1] of
           'B': if chr(v)<>'Z' then blad(zm,41);
           'W': if (chr(v)='T') or (txt='@') then blad(zm,41);
          end;

         ety:=ety+''''+t_mac[indeks+j+5][1]+''''+','+IntToStr(war);

          if j<>k-1 then ety:=ety+',';
        end;

     zm:='@call '+t_mac[indeks+1]+','+ety;

   end else zm:='jsr '+t_mac[indeks+1];

     // wymus wykonanie makra @CALL lub JSR
       i:=1; mne:=oblicz_mnemonik(i,zm,zm);

       ety:='';
    end;


  // .ERROR lub ERT
   if lo(mne) in [__error,$8d] then
    if macro_rept_if_test then
     if pass=pass_end then begin

        save_lst(0,zm,' ');

        omin_spacje(i,zm);

        if zm[i] in ['''','"'] then begin
         if not(run_macro) then writeln(zm) else writeln(old_str);
         txt:=get_string(i,zm,txt,true);
         blad(txt,34)
        end else begin
          txt:=get_dat_noSPC(i,zm,' ',zm);
          war:=oblicz_wartosc(txt,zm);
          if war<>0 then blad(zm,30);
        end;

      end else mne:=__nill;


 // .EXIT
  if lo(mne)=__exit then
   if macro_rept_if_test then begin

      save_lab(ety,adres,bank,zm);
      save_lst(0,zm,' ');

          if run_macro then begin
           wymus_zapis_lst(zm);

           run_macro:=false;
           macro:=false;
//           goto wyjscie;
           wyjscie:=true;
           exit;
          end else if proc then begin
            wymus_zapis_lst(zm);

          // wymus wykonanie makra @EXITPROC
            zm:='@ExitProc '+IntToStr(t_prc[proc_nr-1])+copy(zm,i,length(zm));
            i:=1; mne:=oblicz_mnemonik(i,zm,zm);
            exit_used:=true;
          end;
     ety:='';
    end;


  // .ENDP
  if lo(mne)=__endp then
     if ety<>'' then blad(zm,38) else
      if not(proc) then blad(zm,39) else begin

       save_lst(0,zm,' ');

       if not(exit_used) and (t_prc[proc_nr-1]>0) then begin
        wymus_zapis_lst(zm);

      // wymus wykonanie makra @EXITPROC
        zm:='@ExitProc '+IntToStr(t_prc[proc_nr-1]);
        i:=1; mne:=oblicz_mnemonik(i,zm,zm);
       end;

       oddaj_lokal(zm);

       proc:=false; exit_used:=false; proc_name:='';
      end;


  case lo(mne) of

  // wykonaj .STRUCT
  __struct_run:
    if ety='' then blad(zm,15) else
     if macro_rept_if_test then begin

     save_lab(ety,adres,bank,zm);
     save_lst(0,zm,' ');

     indeks:=mne shr 8;

    // sprawdzamy czy struktura nie wywoluje sama siebie
     if t_str[indeks].lab=lokal_name then blad(zm,57);

     inc(indeks);
     idx:=t_str[indeks].nr;

     while t_str[indeks].nr=idx do begin

      txt:=t_str[indeks].lab;
      _odd:=pos('.',txt);

      txt:=copy(txt,_odd,length(txt));
      str:=ety+txt;

      save_lab(str,t_str[indeks].ofs+adres,bank,zm);

      inc(indeks);
     end;

     ety:='';
    end;


  // wykonaj .MACRO
  __macro_run:
//        if macro then halt {inc(macro_nr)} else
        if if_test then begin

         save_lab(ety,adres,bank,zm);
         save_lst(0,zm,' ');

         indeks:=mne shr 8;

         zapisz_lokal;

         old_macro_nr:=macro_nr;
         str:=t_mac[indeks+1];
         macro_nr:=macro_nr+lokal_name+proc_name+str+t_mac[indeks+3]+'.';

         lokal_name:='';

         omin_spacje(i,zm); SetLength(par,2);
        // na pozycji par[0] zostanie wpisana liczba parametrow

         k:=ord(t_mac[indeks+2][1]);      // tryb pracy
         v:=ord(t_mac[indeks+2][2]);      // separator

         while not(zm[i] in [#0,';','*']) and ((zm[i]<>'/') and (zm[i+1]<>'/')) do begin

          txt:=get_dat(i,zm,chr(v),true);
          if txt<>'' then
           if (txt[1]='"') and (txt[length(txt)]='"') then txt:=copy(txt,2,length(txt)-2);

          _odd:=High(par);

          if k=ord('''') then begin
           par[_odd]:=txt;
           SetLength(par,_odd+2);
          end else begin

            if txt[1]='#' then begin
             par[_odd]:='''#''';
             txt:=copy(txt,2,length(txt))
            end else
             if txt[1] in ['<','>'] then begin
              par[_odd]:='''#''';
              war:=oblicz_wartosc(txt,zm);
              txt:=IntToStr(war);
             end else par[_odd]:=''' ''';

//           _odd:=High(par);
           SetLength(par,_odd+3);
           par[_odd+1]:=txt;
//           SetLength(par,length(par)+1);
          end;

          omin_spacje(i,zm);
           if chr(v)<>' ' then
            if zm[i]=chr(v) then __inc(i,zm) else Break;
//          omin_spacje(i,zm);

         end;

        // zapisujemy liczbe parametrow dla :0
         par[0]:=IntToStr(High(par)-1);

        // zapamietujemy aktualne makro w 'OLD_MACRO_NR'
        // pobieramy numer wywolania makra i zapisujemy w 'MACRO_NR'
        // zwiekszamy numer wywolania makra

         _doo:=StrToInt(t_mac[indeks+3]);
         t_mac[indeks+3]:=IntToStr(_doo+1);

         _doo:=StrToInt(t_mac[indeks+4]);
         txt:=t_mac[indeks];
         save_wyw(zm,txt,_doo);

         _odd:=High(t_mac);

         v:=0;
         while v=0 do begin
          txt:=t_mac[indeks+5];

          j:=1;
          omin_spacje(j,txt);
          ety:=get_datUp(j,txt);

          if ety<>'' then
           if length(ety)=5 then
            if ety[1]='.' then begin
             tst:=oblicz_tst(ety,2);
//             tst:=ord(ety[2]) shl 24+ord(ety[3]) shl 16+ord(ety[4]) shl 8+ord(ety[5]);

//             if ety='.ENDM' then v:=1;
             if tst=$454E444D then v:=1;
            end;

          if v=0 then begin
           j:=1;
           while j<=length(txt) do
            if ((txt[j]=':') and (txt[j+1] in ['0'..'9'])) or ((txt[j]='%') and (txt[j+1]='%') and (txt[j+2] in ['0'..'9'])) then begin
             k:=j;
             if txt[j]=':' then inc(j) else inc(j,2);
             war:=get_value(j,txt,'D',zm);
             if war<0 then blad(zm,47);

             delete(txt,k,j-k); dec(j,j-k);
             if war>High(par)-1 then begin
              insert('$FFFFFFFF',txt,k); inc(j,9);
             end else begin
              insert(par[war],txt,k); inc(j,length(par[war]));
             end;

            end else inc(j);

           save_mac(txt);
          end;

          inc(indeks);
         end;

         txt:=zm; zapisz_lst(txt);

         put_lst(show_full_name(str,false,true));

        // zapamietaj w zmiennych lokalnych
         old_run_macro   := run_macro;
         old_else_used   := else_used;
         old_elseif_used := elseif_used;
         old_ifelse      := ifelse;

         run_macro:=true;

         _doo:=High(t_mac);
         analizuj_mem__(_odd,_doo,zm,a,old_str);

         bez_lst:=true;

         run_macro   := old_run_macro;
         else_used   := old_else_used;
         elseif_used := old_elseif_used;
         ifelse      := old_ifelse;
         macro_nr    := old_macro_nr;

         oddaj_lokal(zm);

         put_lst(show_full_name(a,full_name,true));

         SetLength(t_wyw,length(t_wyw)-1);
         SetLength(t_mac,_odd+1);

         ety:='';
        end;

  // .PROC
   __proc:
        begin
          if ety='' then ety:=get_lab_2(i,zm);

          ety:=lokal_name+ety;

          zapisz_lokal;
          lokal_name:='';

          save_lst(0,zm,'a'); proc_name:='';

          if pass=0 then begin

           reserved_word(ety,zm);

           save_lab(ety,High(t_mac),__id_proc,zm);
           save_mac(a);                         // zapisanie nazwy pliku z procedura
           save_mac(ety);                       // zapisanie nazwy procedury
           str:=IntToStr(bank); save_mac(str);  // zapisanie banku procedury
           str:=IntToStr(adres); save_mac(str); // zapisanie adresu procedury

           indeks:=High(t_mac);                 // zapamietujemy indeks do T_MAC
           str:=' '; save_mac(str);             // liczba parametrow

           inc(proc_nr);

           omin_spacje(i,zm); k:=0; nr_param:=0;

          // odczytujemy deklaracje parametrow spomiedzy { }
           if zm[i]='{' then begin

            proc:=true;
            proc_name:=ety+'.';

           // @STACK_ADDRESS
            txt:='@stack_address'; war:=load_lab(txt,false);
            if war<0 then begin txt:='@STACK_ADDRESS'; war:=load_lab(txt,false) end;

//            if war<0 then begin lokal_name:=txt; blad(zm,5) end;
            war:=t_lab[war].adr;      // odczytujemy wartosc @STACK_ADDRESS

            SetLength(par,1);
            txt:=get_dat(i,zm,',',true); txt:=copy(txt,2,length(txt)-2);

            j:=1;
            while j<=length(txt) do begin
             omin_spacje(j,txt);
             str:=get_datUp(j,txt);

             tst:=0;
             if str<>'' then
              if length(str)=5 then
               if str[1]='.' then
                tst:=oblicz_tst(str,2);
//                tst:=ord(str[2]) shl 24+ord(str[3]) shl 16+ord(str[4]) shl 8+ord(str[5]);

//             if (str<>'.WORD') and (str<>'.BYTE') and (str<>'.LONG') then blad(zm,46);
             if (tst<>$42595445) and (tst<>$574F5244) and (tst<>$4C4F4E47) then blad(zm,46);
             v:=ord(str[2]);

             omin_spacje(j,txt);
             str:=get_dat(j,txt,',',true); omin_spacje(j,txt);

             par[k]:=str;
             SetLength(par,k+2);

             save_lab(str,nr_param+war,__id_param,zm);

             str:=chr(v)+str; save_mac(str);

             inc(k);

              case chr(v) of
               'B': inc(nr_param,1);
               'W': inc(nr_param,2);
               'L': inc(nr_param,3);
              end;

            end;    // while

            if k>0 then
             for j:=0 to k-1 do begin
              _odd:=load_lab(par[j],true);
              war:=t_lab[_odd].adr;
              save_lab(par[j],war-nr_param,__id_param,zm);
             end;

           end;     // if zm[i]='{'

           dec(proc_nr);
           t_prc[proc_nr]:=nr_param;

           t_mac[indeks]:=IntToStr(k);
          end else begin                // if pass=0 then

         // uaktualniamy wartosc BANK i ADRES dla procedury
           _odd:=load_lab(ety,true);

            indeks:=t_lab[_odd].adr;
            t_mac[indeks+2]:=IntToStr(bank);
            t_mac[indeks+3]:=IntToStr(adres);

            str:=lokal_name+ety;
            zapisz_etykiete(str,adres,bank,ety[1]);
          end;

          proc:=true;
          inc(proc_nr);

          if proc_nr>High(t_prc) then SetLength(t_prc,proc_nr+1);

          proc_name:=ety+'.';
          ety:=''
         end;

  // .MACRO
   __macro:
         begin
          if ety='' then ety:=get_lab_2(i,zm);

          if adres<0 then adres:=$0100;

          if pass=0 then begin

           reserved_word(ety,zm);

           save_lab(ety,High(t_mac),__id_macro,zm);
           save_mac(a);                        // zapisanie nazwy pliku z makrem
           str:=lokal_name+ety; save_mac(str); // zapisanie nazwy makra

           omin_spacje(i,zm);
           ety:=''''; txt:=',';

           omin_spacje(i,zm);
           if zm[i] in ['''','"'] then begin
           // wczytujemy i sprawdzamy poprawnosc
            ety:=zm[i];
            txt:=get_string(i,zm,zm,true);
            if length(txt)=0 then txt:=',' else
             if length(txt)>1 then blad(zm,3);
           end;

           str:=ety+txt; save_mac(str);        // separator i tryb dzialania
           str:='0'; save_mac(str);            // numer wywolania makra
           str:=IntToStr(line); save_mac(str); // numer linii z makrem
          end;

          inc(dek_macro_nr);
          macro:=true;

          save_lst(0,zm,' ');

          ety:=''
         end;

  // .LOCAL
   __local:
        if macro_rept_if_test then begin

          if ety='' then ety:=get_lab_2(i,zm);

          save_lab(ety,adres,bank,zm);
          save_lst(0,zm,'a');

          zapisz_lokal;
          lokal_name:=lokal_name+ety+'.';

          ety:=''
         end;

  // .STRUCT
   __struct:
        if macro_rept_if_test then
         if struct then blad(zm,56) else begin

          if ety='' then ety:=get_lab_2(i,zm);

          save_lab(ety,High(t_str),__id_struct,zm);
          save_lst(0,zm,'a');

          struct:=true;
          struct_adres:=adres;

          inc(struct_nr);

          zapisz_lokal;
          lokal_name:=lokal_name+ety+'.';

          save_str(lokal_name,adres);       // nazwa struktury

          ety:=''
         end;

  // .IF
   __if:
        if ety<>'' then blad(zm,38) else
         if not(macro) and not(rept) then begin

           if_stos[ifelse]._else    := else_used;
           if_stos[ifelse]._elseif  := elseif_used;
           if_stos[ifelse]._if_test := if_test;

           inc(ifelse);
           if ifelse>High(if_stos) then SetLength(if_stos,ifelse+1);

           if_stos[ifelse]._okelse:=0;

           if if_test then begin
            save_lst(0,zm,' ');

            else_used   := false;
            elseif_used := false;

            if_stos[ifelse]._okelse:=ifelse;

            txt:=get_dat_noSPC(i,zm,' ',zm);

            war:=oblicz_wartosc(txt,zm);

//            if war<>0 then if_test:=true else if_test:=false;
            if_test:=(war<>0);
           end;

           if_stos[ifelse]._else    := else_used;
           if_stos[ifelse]._elseif  := elseif_used;
           if_stos[ifelse]._if_test := if_test;
         end;

  // .ELSEIF
   __elseif:
       if ety<>'' then blad(zm,38) else
        if not(macro) and not(rept) then
         if if_test and else_used then blad(zm,42) else
           if ifelse=0 then blad(zm,37) else
            if not(if_test) and not(elseif_used) then begin

              save_lst(0,zm,' ');

              omin_spacje(i,zm);
              war:=oblicz_wartosc(copy(zm,i,len),zm);

              if war<>0 then begin
               elseif_used:=true;
               if_test:=true
              end else begin
               elseif_used:=false;
               if_test:=false
              end;

              if_stos[ifelse]._if_test := if_test;

            end else if_test:=false;

  // .ENDIF
   __endif:
        if ety<>'' then blad(zm,38) else
         if not(macro) and not(rept) then
          if ifelse>0 then begin

           dec(ifelse);

           if_test     := if_stos[ifelse]._if_test;
           else_used   := if_stos[ifelse]._else;
           elseif_used := if_stos[ifelse]._elseif;

//           if ifelse=0 then if_test:=true;

          end else blad(zm,37);

  // .ENDM
   __endm:
        if ety<>'' then blad(zm,38) else
         if not(macro) then blad(zm,35) else begin
          macro:=false;
          save_mac(zm);
         end;

  // .ENDL
   __endl:
        if ety<>'' then blad(zm,38) else
         if macro_rept_if_test then
          if lokal_nr>0 then begin

           save_lst(0,zm,' ');

           oddaj_lokal(zm);

          end else blad(zm,28);

  // .ENDS
   __ends:
        if ety<>'' then blad(zm,38) else
         if macro_rept_if_test then
          if struct then begin

           save_lst(0,zm,' ');

           oddaj_lokal(zm);

           struct:= false;
           adres := struct_adres;

          end else blad(zm,55);

  // .ELSE
   __else:
       if ety<>'' then blad(zm,38) else
        if not(macro) and not(rept) then
         if if_stos[ifelse]._okelse=ifelse then begin

          if else_used then blad(zm,1);
          if ifelse=0 then blad(zm,37);

          if_stos[ifelse]._else:=true;
          else_used:=true;

//          if_test:=not(if_test);
          if_test := not(if_stos[ifelse]._if_test);
         end;

  // .PRINT
   __print:
       if ety<>'' then blad(zm,38) else
        if macro_rept_if_test then
         if pass=pass_end then begin

           save_lst(0,zm,' ');

           pisz:=true;

           omin_spacje(i,zm);
           oblicz_dane(i,zm,zm,4);

           pisz:=false;
           writeln;
          end;

  // EQU
   $80,$92:
       if ety='' then blad(zm,15) else
        if macro_rept_if_test then
         if loop_used then blad(zm,36) else begin

          // SMB
           if lo(mne)=$92 then begin

            txt:=get_smb(i,zm);

          // wymuszamy relokowalnosc dla tej etykiety
          // jesli BLOK>1 to zaznaczy jako etykiete relokowalna
            k:=blok; blok:=$FFFF;
            save_lab(ety,High(t_smb),__id_smb,zm);
            blok:=k;

            save_smb(txt);
            war:=__rel;

           end else begin
            branch:=true;       // dla EQU nie ma relokowalnosci

            txt:=get_dat_noSPC(i,zm,' ',zm);
            war:=oblicz_wartosc(txt,zm);

            save_lab(ety,war,bank,zm);
           end;

           save_lst(war,zm,'l');
           ety:='';
          end;

  // OPT (bit)
  // 0 - Header
  // 1 - Object file
  // 2 - Listing
  // 3 - Screen (listing on screen)
  // 4 - CPU 8bit/16bit
   $81: if macro_rept_if_test then begin

         omin_spacje(i,zm);
         txt:=get_dat(i,zm,',',true);

         j:=1; v:=0; dat:=false;
         while j<=length(txt) do begin

          case UpCase(txt[j]) of
           'H': begin
                 v:=test_opt(txt[j+1],v,1);
                 if txt[j+1]='-' then opt_h_minus($ff);
                end;
           'O': v:=test_opt(txt[j+1],v,2);
           'L': v:=test_opt(txt[j+1],v,4);
           'S': v:=test_opt(txt[j+1],v,8);
           'C': v:=test_opt(txt[j+1],v,16);
          else
           dat:=true;
          end;

          if dat then blad(zm,16);
          inc(j,2);
         end;

         save_lst(0,zm,' ');
         dat:=false;
        end;

  // ORG,RUN,INI
   $82,$87,$89,$d0..$d2:
    if not(macro) then
     if if_test then
      if loop_used or rept then blad(zm,36) else
       if first_org and (ety<>'') then blad(zm,10) else begin

         save_lst(0,zm,'a');

         branch:=true;    // relokowalnosc tutaj niemozliwa

         rel_ofs:=0;
         idx:=-$FFFF; hea_ofs:=-1; _odd:=-1;

         save_lab(ety,adres,bank,zm);

         omin_spacje(i,zm);

         v:=lo(mne);
         case v of
         // BLK SPARTA a
          $d0: begin
                k:=opt;
                opt_h_minus($fe);

                blok:=0;

              // a($fffa),a(str_adr),a(end_adr)
                save_dst($fa); save_dst($ff);

                idx:=oblicz_wartosc(get_dat(i,zm,',',true),zm);
                opt:=k;
               end;

         //BLK R[eloc] M[ain]|E[xtended]
          $d1: begin
                k:=opt;
                opt_h_minus($fe);

                txt:=get_ciag(i,zm);

                case UpCase(txt[1]) of
              // M[ain]
                'M': v:=0;

              // E[xtended]
                'E': v:=2;
                else
                 blad(zm,23);
                end;

                add_blok(zm);

              // a($fffe),b(blk_num),b(blk_id)
              // a(blk_off),a(blk_len)
                save_dst($fe); save_dst($ff);
                save_dst(blok);save_dst(v);

                if blok=1 then
                 idx:=__rel
                else begin
                 rel_ofs:=adres-__rel;
                 _odd:=adres; idx:=adres;
                end;

                opt:=k;
               end;

         //BLK E[mpty] a M[ain]|E[xtended]
          $d2: begin
                k:=opt;
                opt_h_minus($fe);

                idx:=oblicz_wartosc(get_dat(i,zm,',',true),zm);

                txt:=get_ciag(i,zm);

                case UpCase(txt[1]) of
              // M[ain]
                'M': v:=$80;

              // E[xtended]
                'E': v:=$82;
                else
                 blad(zm,23);
                end;

                add_blok(zm);

              // a($fffe),b(blk_num),b(blk_id)
              // a(blk_off),a(blk_len)
                save_dst($fe); save_dst($ff);
                save_dst(blok); save_dst(v);

                save_dst(lo(adres)); save_dst(hi(adres));
                save_dst(lo(idx)); save_dst(hi(idx));

               // jesli deklaracja etykiet wystapila przed blokiem EMPTY
               // znajdz je i popraw im numer bloku
                indeks:=adres; tst:=adres+idx;
                for _doo:=0 to High(t_lab)-1 do
                 if (t_lab[_doo].adr>=indeks) and (t_lab[_doo].adr<=tst) then
                  t_lab[_doo].blk:=blok;

                inc(adres,idx);
                idx:=adres;

                opt:=k;
               end;

          // ORG
          $82: begin
                if zm[i]='[' then begin
                 v:=opt;
                 opt_h_minus($fe);

                 txt:=ciag_ograniczony(i,zm,'[',']',true);
                 k:=1; oblicz_dane(k,txt,zm,1);

                 omin_spacje(i,zm);
                 if zm[i]=',' then begin
                  inc(i);
                  txt:=get_dat_noSPC(i,zm,' ',zm);
                  idx:=oblicz_wartosc(txt,zm);
                 end;
                 opt:=v;

                end else begin
                 txt:=get_dat_noSPC(i,zm,',',zm);
                 idx:=oblicz_wartosc(txt,zm);
                end;

                 omin_spacje(i,zm);
                 if zm[i]=',' then begin
                  inc(i);
                  txt:=get_dat_noSPC(i,zm,' ',zm);
                  hea_ofs:=oblicz_wartosc(txt,zm);
                 end;

               end;

          // RUN,INI
          $87,$89:
               begin
                idx:=oblicz_wartosc(copy(zm,i,len),zm);
                mne:=idx shl 8+2; idx:=$2e0+v-$87;
               end;
         end;

         if (adres<>idx) or (_odd>=0) then begin
          t_hea[hea_i]:=adres-1;
          inc(hea_i);
          if hea_i>High(t_hea) then SetLength(t_hea,hea_i+1);
          adres:=idx; org:=true;
         end;

         if idx>=0 then first_org:=false;
         ety:='';
        end;

  // INS
   $83:
     if macro_rept_if_test then
      if loop_used then blad(zm,36) else
       if first_org and (opt and 1>0) then blad(zm,10) else begin

         omin_spacje(i,zm);
         txt:=get_string(i,zm,zm,true);
         txt:=GetFile(txt);

//         if not(FileExists(txt)) then blad(zm,18);

//         AssignFile(g, txt);
//         {$I-}
//         Reset(g,1);
//         {$I+}
//         IORes:=IOResult;
//         if not((IORes=32) or (IORes=0)) then blad(zm,18);

         if not(TestFile(txt)) then blad(zm,18);
         if (GetFileName(txt)=GetFileName(a)) or (GetFileName(txt)=GetFileName(plik_asm)) then blad(zm,18);

         AssignFile(g, txt);
         Reset(g,1);

         war:=FileSize(g); _odd:=0; _doo:=war;

         v:=test_string(i,zm);

         if zm[i]=',' then begin
          __inc(i,zm);
          _odd:=oblicz_wartosc(get_dat_noSPC(i,zm,',',zm),zm);
          if _odd<0 then begin
           _doo:=abs(_odd); _odd:=war+_odd;
          end else _doo:=war-abs(_odd);

          if zm[i]=',' then begin
           __inc(i,zm);
           _doo:=oblicz_wartosc(get_dat(i,zm,',',true),zm);
           if _doo<0 then blad(zm,0);
           test_eol(i,zm,zm,#0);
          end;
         end else test_eol(i,zm,zm,#0);

        // sprawdz czy nie zostala przekroczona dlugosc pliku
         if (abs(_odd)>war) or (abs(_odd)+abs(_doo)>war) then blad(zm,24);
         war:=_doo;

        // maksymalna dlugosc pliku to 64kB
         if war>$FFFF then blad(zm,25);

         if (pass=pass_end) and (_doo>0) then begin
           save_lst(0,zm,'a');

          // omin '_ODD' bajtow w pliku
           if _odd>0 then seek(g,_odd);

          // zapisz '_DOO' bajtow
           blockread(g,t_ins,_doo);
           for i:=0 to _doo-1 do save_dst(t_ins[i]+v);
         end;
         closefile(g);

         save_lab(ety,adres,bank,zm);
         inc(adres,war);
         ety:='';
        end;

  // END
   $84: if loop_used then blad(zm,36) else begin
         end_file:=true;
         save_lab(ety,adres,bank,zm);
         save_lst(0,zm,'a');
         ety:='';
        end;

  // DTA
   $85,__byte..__dword:
        if macro_rept_if_test then
         if first_org and (opt and 1>0) then blad(zm,10) else begin

           save_lab(ety,adres,bank,zm);
           save_lst(0,zm,'a');

           v := lo(mne);
           if v>=__byte then v:=v-__byte+1 else v:=1;

           omin_spacje(i,zm);
           oblicz_dane(i,zm,zm,v);
           ety:='';
          end;

  // ICL
   $86:
       if macro_rept_if_test then
        if loop_used then blad(zm,36) else begin

          save_lab(ety,adres,bank,zm);
          save_lst(0,zm,'a');

          omin_spacje(i,zm); txt:=get_string(i,zm,zm,true);
          txt:=GetFile(txt);
          test_eol(i,zm,zm,#0);

//         if not(FileExists(txt)) or (txt=a) then blad(zm,18);
          if (GetFileName(txt)=GetFileName(a)) or (GetFileName(txt)=GetFileName(plik_asm)) then blad(zm,18);

          justuj; put_lst(t+zm);

          old_icl_used := icl_used;
          icl_used:=true;

          analizuj_plik__(txt,zm);

          bez_lst:=true;
          put_lst(show_full_name(a,full_name,true));
          global_name:=a;

          line:=nr;

          icl_used := old_icl_used;

          ety:='';
         end;

  // NMB
   $88: if macro_rept_if_test then begin
         inc(bank);
         save_lst(0,zm,'i');
        end;

  // RMB
   $8b: if macro_rept_if_test then begin
         bank:=0;
         save_lst(0,zm,'i');
        end;

  // LMB
   $8c: if macro_rept_if_test then begin

         omin_spacje(i,zm);
         if zm[i] in ['<','>'] then else
          if zm[i]<>'#' then blad(zm,14) else __inc(i,zm);

         j:=oblicz_wartosc(get_dat(i,zm,',',true),zm);
         if j<__id_param then bank:=j else blad(zm,0);

         save_lst(0,zm,'i');
        end;

  // .REPT
   __rept:
       if not(macro) then
        if if_test then
         if rept then blad(zm,43) else begin

         save_lab(ety,adres,bank,zm);
         save_lst(0,zm,' ');

         omin_spacje(i,zm);
         rept_ile:=oblicz_wartosc(copy(zm,i,len),zm);
         if rept_ile<0 then blad(zm,0);

         rept_line:=line;

         rept_nr:=High(t_mac);
         rept:=true;

         ety:='';
        end;

  // .ENDR
   __endr:
     if not(macro) then
      if if_test then
       if not(rept) then blad(zm,51) else begin

         wymus_zapis_lst(zm);

         rept:=false;
         idx:=High(t_mac)-1;

         old_icl_used := icl_used;
         icl_used     := true;

         txt:='REPT';
         put_lst(show_full_name(txt,false,true));

         for _odd:=0 to rept_ile-1 do
          analizuj_mem__(rept_nr,idx,zm,a,old_str);

//         save_lab(ety,adres,bank,zm);

         bez_lst  := true;
         icl_used := old_icl_used;

         put_lst(show_full_name(a,full_name,true));
         line:=nr;
         rept_line:=0;

         SetLength(t_mac,rept_nr+1);

         ety:='';
       end;

  end;

 // zapamietanie etykiety jesli wystapila
 // i zapisanie zdekodowanych rozkazow
  if if_test then begin

   if ety<>'' then begin
    if pass=pass_end then
     if adres<0 then warning(10);

    save_lab(ety,adres,bank,zm);
    if lo(mne)=0 then save_lst(0,zm,'a');
   end;

   if lo(mne)<$80 then save_lst(mne,zm,'a');
  end;

  dat:=false;

 // zapisz wynik asemblacji do pliku LST
 if not(loop_used) then begin
  justuj;

  if not(bez_lst) then put_lst(t+zm);
  bez_lst:=false;
 end;

end;



procedure get_linia(var zm:string; var end_file,ok:Boolean);
var i, j: word;
    str: string;
begin
  if loop_used and (zm='') then zm:=' ';

  i:=1;
  if not(end_file) then
   if zm='' then begin
    save_lst(0,zm,';');
    str:=' ';
    if (pass=0) and macro then save_mac(str) else
     if rept then save_mac(str);
   end else begin
    omin_spacje(i,zm);
    if (zm[i] in [';','*','/']) or komentarz then begin
     save_lst(0,zm,';');
     if (pass=0) and macro then save_mac(zm) else
      if rept then save_mac(zm);

     case zm[i] of
      '*': if komentarz then
            if zm[i+1]='/' then komentarz:=false else begin
             j:=pos('*/',zm);
             if (j>0) and (zm[j+2] in [#0,' ']) then komentarz:=false;
            end;

      '/': if zm[i+1]='*' then komentarz:=true else
            if zm[i+1]<>'/' then ok:=true;
     end;

     j:=pos('*/',copy(zm,i,length(zm)));
     if (j>0) and (zm[j+2] in [#0,' ']) then komentarz:=false;

    end else ok:=true;

   end;
end;


procedure test_wyjscie(var zm:string; var wyjscie:Boolean);
begin
 if not(wyjscie) then begin

  if not(run_macro) and not(icl_used) then
   if ifelse<>0 then blad(zm,1) else
    if proc or macro then blad(zm,17) else
     if struct then blad(zm,56) else
      if rept then blad(zm,51) else
       if lokal_nr>0 then blad(zm,29);

  if run_macro and macro then blad(zm,17);

 end;
end;


procedure analizuj_plik(var a:string; old_str: string);
// odczyt pliku ASM wiersz po wierszu
var f: textfile;
    rept_nr, rept_ile, nr: integer;
    end_file, ok, wyjscie: Boolean;
    zm: string;

begin
// AssignFile(f, a);
// {$I-}
// Reset(f);
// {$I+}
// IORes:=IOResult;
// if not((IORes=32) or (IORes=0)) then blad(old_str,18);

 if not(TestFile(a)) then blad(old_str,18);

 AssignFile(f,a);
 Reset(f);

 nr:=0;                        // nr linii w asemblowanym pliku
 global_name:=a;               // nazwa pliku dla funkcji 'BLAD'
 end_file:=false;              // czy napotkal rozkaz 'END'
 wyjscie:=false;               // czy wystapilo EXIT w makrze

 // nazwe pierwszego pliku zawsze zapisze do LST
 if (pass=pass_end) and (a[2]=':') then writeln(lst,show_full_name(global_name,true,true))
  else put_lst(show_full_name(global_name,full_name,true));

 while not eof(f) do begin

  ok:=false;

  readln(f,zm);
  inc(nr); line:=nr; inc(line_all);

  formatuj(zm);

  get_linia(zm,end_file,ok);

  if ok and not(end_file) and not(komentarz) then
   analizuj_linie(zm,old_str,a, rept_nr,rept_ile, nr, end_file,wyjscie);

 end;

 test_wyjscie(zm,wyjscie);

 closefile(f);
end;



procedure analizuj_mem(start,koniec:integer; var old, a, old_str:string);
// odczyt i analiza z pamieci RAM
var rept_ile, rept_nr, licznik, nr: integer;
    end_file, ok, wyjscie: Boolean;
    zm: string;

begin
 licznik:=start;

 nr:=0;                        // nr linii w asemblowanym pliku
 end_file:=false;              // czy napotkal rozkaz 'END'
 wyjscie:=false;               // czy wystapilo EXIT w makrze

 while licznik<koniec do begin

  ok:=false;

  zm:=t_mac[licznik]; inc(licznik);
  inc(nr); line:=nr; inc(line_all);

  formatuj(zm);

  get_linia(zm,end_file,ok);

  if ok and not(end_file) and not(komentarz) then
   analizuj_linie(zm,old_str,a, rept_nr,rept_ile, nr, end_file,wyjscie);

 end;

 test_wyjscie(old,wyjscie);
end;



procedure asem(var a:string);
// asemblacja glownego pliku *.ASM
var i: integer;
begin

 while pass<=pass_end do begin

  line_all:=0;

  analizuj_plik(a,'');

  if list_mac then analizuj_plik(plik_mac,' icl '''+plik_mac+'''');

  t_hea[hea_i]:=adres-1;    // koniec programu, ostatni wpis

  inc(pass);

 // wyzeruj numer wywolania makra
  for i:=0 to High(t_lab) do
   if t_lab[i].bnk=__id_macro then begin
    line:=t_lab[i].adr;
    t_mac[line+3]:='0';
   end;

  adres:=-$FFFF; hea_ofs:=-1;

  hea_i:=0; bank:=0; ifelse:=0; blok:=0; rel_ofs:=0;
  proc_nr:=0; dek_macro_nr:=0; lokal_nr:=0;
  rept_line:=0; struct_nr:=0;

  opt:=opt_old;

  hea:=true; first_org:=true; if_test:=true;

  loop_used:=false; macro:=false; proc:=false;
  else_used:=false; elseif_used:=false; exit_used:=false;
  icl_used:=false; bez_lst:=false; empty:=false;
  reloc:=false; branch:=false; vector:=false; rept:=false;
  struct:=false; dta_used:=false; label_used:=false;

  komentarz:=false;

  lokal_name:=''; macro_nr:='';

  SetLength(t_loc,1);
  SetLength(t_wyw,1);
  SetLength(t_rel,1);
  SetLength(t_smb,1);

 end;
end;


procedure Syntax;
// wyswietla instrukcje uzytkowania
begin
 writeln(load_mes(59+1));
 halt(3);
end;


function nowy_plik(var a: string):string;
begin
 Result:=a;

 name:=get_dat(help,t,',',true);
 if GetFilePath(name)<>'' then path:=GetFilePath(name);
 if name<>'' then Result:=path+GetFileName(name);
end;


procedure calc_crc;
var x, y: byte;
    crc: cardinal;
    crc_: Int64;
begin

for x:=0 to 255 do begin

 crc:=x shl 8;
 crc_:=crc;

 for y:=1 to 8 do begin
  crc := crc shl 1;
  crc_ := crc_ shr 1;

  if (crc and $00010000) > 0 then crc := crc xor $1021;
  if (crc_ and $80) > 0 then crc_ := crc_ xor $edb8832000;
 end;

 tcrc16[x] := word(crc);
 tcrc32[x] := cardinal(crc_ shr 8);
end;
end;


{
procedure tttt(a: string);
begin
 case length(a) of
  2: writeln(hex(ord(a[1]) shl 8+ord(a[2]),8));
  3: writeln(hex(ord(a[1]) shl 16+ord(a[2]) shl 8+ord(a[3]),8));
  4: writeln(hex(ord(a[1]) shl 24+ord(a[2]) shl 16+ord(a[3]) shl 8+ord(a[4]),8));
 end;
end;
}


// -------------- //
//  MAIN PROGRAM  //
// -------------- //
begin
 oblicz_mnemonik__:=oblicz_mnemonik;
 analizuj_plik__:=analizuj_plik;
 analizuj_mem__:=analizuj_mem;

 FileMode:=0;

 SetLength(t_lab,1);
 SetLength(t_hea,1);
 SetLength(t_mac,1);
 SetLength(t_loc,1);
 SetLength(t_prc,1);
 SetLength(t_pth,1);
 SetLength(t_wyw,1);
 SetLength(t_rel,1);
 SetLength(t_smb,1);
 SetLength(t_str,1);
 SetLength(if_stos,1);

{
// wyliczamy wartosc dla n/w ciagow aby skrocic IF'a i program
 tttt('ENDR');
 tttt('ENDM');
 tttt('BYTE');
 tttt('WORD');
 tttt('LONG');
 halt;
}


{
// wyliczamy wartosc dla n/w ciagow aby skrocic IF'a i program
 tttt('ASL');
 tttt('LSR');
 tttt('ROL');
 tttt('ROR');
 tttt('NO');
 tttt('DE');
 tttt('ND(');
 tttt('IN(');
 tttt('OR');
 tttt('XOR');
 tttt('AND');
 tttt('MVP');
 tttt('MVN');
 tttt('PEA');
 tttt('COP');
 tttt('REP');
 tttt('SEP');
 halt;
}


{
// wyliczamy wartosc dla n/w ciagow i wstawiamy
// do procedury 'OperExt', zamiast IF'ow mamy CASE
 tttt('<>'); tttt('!='); // A
 tttt('<=');             // B
 tttt('>=');             // C
 tttt('<<');             // D
 tttt('>>');             // E
 tttt('==');             // =
 tttt('&&');             // F
 tttt('||');             // G

 halt;
}


 // tworzenie tablic TCRC16, TCRC32
 calc_crc;

 // tworzenie tablicy HASH
 for IORes:=0 to 165 do hash[hash_ofs[IORes]]:=hash_val[IORes];


 if paramcount<1 then Syntax;

 t:=ParamStr(1); if t[1]='/' then Syntax;

 path:=GetFilePath(t);
 if path='' then begin GetDir(0,path); path:=path+'\' end;

 name:=GetFileName(t);
 global_name:=name;

// if not(FileExists(path+name)) then blad('',18);

// AssignFile(temp_file, path+name);
// {$I-}
// Reset(temp_file);
// {$I+}
// IORes:=IOResult;
// if not((IORes=32) or (IORes=0)) then begin t:=''; blad(t,18) end;
// CloseFile(temp_file);

 plik_lst:=path+name;
 if not(TestFile(plik_lst)) then begin t:=''; blad(t,18) end;

 plik_lst:=path+NewFileExt(name,0);
 plik_obj:=path+NewFileExt(name,1);
 plik_lab:=path+NewFileExt(name,2);
 plik_mac:=path+NewFileExt(name,3);
 plik_h:=path+NewFileExt(name,4);
 plik_asm:=path+name;

 // odczyt parametrow
 for IORes:=2 to ParamCount do begin
  t:=ParamStr(IORes);

  if t[1]<>'/' then Syntax;

  help:=4;
  case UpCase(t[2]) of
   'C': if t[3]<>'' then Syntax else case_used:=true;

   'E': begin white_spaces:=true; if t[3]<>'' then Syntax end;

   'H': begin
         list_hhh:=true;
         if t[3]=':' then plik_h:=nowy_plik(plik_h);
        end;

   'I': if t[3]<>':' then Syntax else begin
         delete(t,1,3); name:=t;

         if name='' then Syntax else begin
          t_pth[High(t_pth)]:=name;
          SetLength(t_pth,length(t_pth)+1);
         end;

        end;

   'L': begin
         opt:=opt or 4;
         if t[3]=':' then plik_lst:=nowy_plik(plik_lst);
        end;

   'M': if t[3]<>':' then Syntax else begin
         list_mac:=true;
         plik_mac:=nowy_plik(plik_mac);
        end;

   'O': if t[3]<>':' then Syntax else plik_obj:=nowy_plik(plik_obj);

   'P': if (t[3]<>' ') and (t[3]<>'') then Syntax else full_name:=true;

   'S': begin silent:=true; if t[3]<>'' then Syntax end;
   
   'T': begin
         list_lab:=true;
         if t[3]=':' then plik_lab:=nowy_plik(plik_lab);
        end;

  else
   Syntax;
  end;
 end;


 if not(silent) then writeln(load_mes(50+1));

 if list_lab then begin
  assignfile(lab,plik_lab); rewrite(lab);
  writeln(lab,load_mes(50+1));
  writeln(lab,load_mes(54+1));
 end;

 if list_hhh then begin
  name:=GetFileName(plik_asm);
  help:=1; name:=get_dat(help,name,'.',true);

  assignfile(hhh,plik_h); rewrite(hhh);
  writeln(hhh,'#ifndef _'+name+'_ASM_H_');
  writeln(hhh,'#define _'+name+'_ASM_H_'+#13+#10);
 end;

 assignfile(lst,plik_lst); rewrite(lst);
 assignfile(dst,plik_obj); rewrite(dst,1);
 writeln(lst,load_mes(50+1)); open_ok:=true;

 opt_old:=opt; t:='';

 asem(plik_asm);

 over:=true;
 koniec(0);
end.
